import platform

# use relative import here and not absolute, so that `python -m docs` works at
# development time
from .serve import main as serve
from .static import main as static


def main():
    if platform.system() == "Linux":
        serve()
    else:
        static()


if __name__ == "__main__":
    main()



# python -m pygame.docs

from pathlib import Path


PKG_DIR = Path(__file__).absolute().parent / "generated"


# for test suite to confirm pygame-ce built with local docs
def has_local_docs():
    return PKG_DIR.exists()



#!/usr/bin/env python
"""pygame.examples.headless_no_windows_needed

How to use pygame-ce with no windowing system, like on headless servers.

Thumbnail generation with scaling is an example of what you can do with pygame.
NOTE: the pygame scale function uses mmx/sse if available, and can be run
  in multiple threads.
"""

usage = """-scale inputimage outputimage new_width new_height
eg.  -scale in.png out.png 50 50

"""

import os
import sys
import pygame

# set SDL to use the NULL video driver,
#   so it doesn't need a windowing system.
os.environ["SDL_VIDEODRIVER"] = pygame.NULL_VIDEODRIVER


# Some platforms need to init the display for some parts of pygame.
pygame.display.init()
screen = pygame.display.set_mode((1, 1))


def scaleit(fin, fout, w, h):
    i = pygame.image.load(fin)
    scaled_image = pygame.transform.smoothscale(i, (w, h))
    pygame.image.save(scaled_image, fout)


def main(fin, fout, w, h):
    """smoothscale image file named fin as fout with new size (w,h)"""
    scaleit(fin, fout, w, h)


if __name__ == "__main__":
    if "-scale" in sys.argv:
        fin, fout, w, h = sys.argv[2:]
        w, h = map(int, [w, h])
        main(fin, fout, w, h)
    else:
        print(usage)



import math
import pygame


def new_canvas():
    # Return a new blank canvas.
    return pygame.Surface((320, 240))


def main():
    pygame.init()
    clock = pygame.Clock()

    # Screen is a resizable window.
    screen = pygame.display.set_mode((320, 240), pygame.RESIZABLE)

    # Canvas is the surface to draw to.
    canvas = new_canvas()

    pygame.display.set_caption("Pygame - Retro scaling")

    running = True

    while running:
        for event in pygame.event.get():
            if event.type == pygame.QUIT:
                running = False

        # A fresh canvas to work on.
        canvas = new_canvas()

        # Fill the screen-surface with a background-color.
        screen.fill((128, 0, 0))

        # Fill the canvas-surface with a background-color.
        canvas.fill((0, 128, 0))

        # Render a circle to demonstrate the scaling.
        pygame.draw.circle(canvas, (255, 255, 255), (160, 120), 32)

        # Calculate the scale-factor for the canvas-surface.
        scale_width = math.floor(screen.get_width() / canvas.get_width())
        scale_height = math.floor(screen.get_height() / canvas.get_height())

        # Set scale to the smallest of width and height.
        scale = min(scale_width, scale_height)

        # Set scale to a minimum of 1.
        scale = max(scale, 1)

        # Scale the canvas-surface.
        canvas = pygame.transform.scale_by(canvas, scale)

        # Calculate the center for the screen-surface.
        screen_center_x = math.floor(screen.get_width() / 2)
        screen_center_y = math.floor(screen.get_height() / 2)

        # Calculate the center for the canvas-surface.
        canvas_center_x = math.floor(canvas.get_width() / 2)
        canvas_center_y = math.floor(canvas.get_height() / 2)

        # Calculate the new canvas-position.
        canvas_x = screen_center_x - canvas_center_x
        canvas_y = screen_center_y - canvas_center_y

        # Render the canvas-surface in the center of screen-surface.
        screen.blit(canvas, (canvas_x, canvas_y))

        pygame.display.flip()

        clock.tick(30)

    pygame.quit()


if __name__ == "__main__":
    main()



from collections.abc import Callable, Hashable, Iterable
from typing import Literal, Optional, Union
from typing_extensions import deprecated  # added in 3.13

from pygame.surface import Surface

from pygame.typing import ColorLike, FileLike

# TODO: Figure out a way to type this attribute such that mypy knows it's not
# always defined at runtime
UCS4: Literal[1]

def init() -> None: ...
def quit() -> None: ...
def get_init() -> bool: ...
def get_sdl_ttf_version(linked: bool = True) -> tuple[int, int, int]: ...
def get_default_font() -> str: ...
def get_fonts() -> list[str]: ...
def match_font(
    name: Union[str, bytes, Iterable[Union[str, bytes]]],
    bold: Hashable = False,
    italic: Hashable = False,
) -> str: ...
def SysFont(
    name: Union[str, bytes, Iterable[Union[str, bytes]], None],
    size: int,
    bold: Hashable = False,
    italic: Hashable = False,
    constructor: Optional[Callable[[Optional[str], int, bool, bool], Font]] = None,
) -> Font: ...

class Font:
    @property
    def name(self) -> str: ...
    @property
    def style_name(self) -> str: ...
    @property
    def bold(self) -> bool: ...
    @bold.setter
    def bold(self, value: bool) -> None: ...
    @property
    def italic(self) -> bool: ...
    @italic.setter
    def italic(self, value: bool) -> None: ...
    @property
    def underline(self) -> bool: ...
    @underline.setter
    def underline(self, value: bool) -> None: ...
    @property
    def strikethrough(self) -> bool: ...
    @strikethrough.setter
    def strikethrough(self, value: bool) -> None: ...
    @property
    def align(self) -> int: ...
    @align.setter
    def align(self, value: int) -> None: ...
    @property
    def point_size(self) -> int: ...
    @point_size.setter
    def point_size(self, value: int) -> None: ...
    def __init__(self, filename: Optional[FileLike] = None, size: int = 20) -> None: ...
    def render(
        self,
        text: Union[str, bytes, None],
        antialias: bool,
        color: ColorLike,
        bgcolor: Optional[ColorLike] = None,
        wraplength: int = 0,
    ) -> Surface: ...
    def size(self, text: Union[str, bytes], /) -> tuple[int, int]: ...
    def set_underline(self, value: bool, /) -> None: ...
    def get_underline(self) -> bool: ...
    def set_strikethrough(self, value: bool, /) -> None: ...
    def get_strikethrough(self) -> bool: ...
    def set_bold(self, value: bool, /) -> None: ...
    def get_bold(self) -> bool: ...
    def set_italic(self, value: bool, /) -> None: ...
    def metrics(
        self, text: Union[str, bytes], /
    ) -> list[tuple[int, int, int, int, int]]: ...
    def get_italic(self) -> bool: ...
    def get_linesize(self) -> int: ...
    def get_height(self) -> int: ...
    def get_ascent(self) -> int: ...
    def get_descent(self) -> int: ...
    def set_script(self, script_code: str, /) -> None: ...
    def set_direction(self, direction: int) -> None: ...
    def get_point_size(self) -> int: ...
    def set_point_size(self, val: int, /) -> None: ...

@deprecated("Use `Font` instead (FontType is an old alias)")
class FontType(Font): ...



from collections.abc import Collection, Iterator
from typing import Any, SupportsIndex, Union, overload
from typing_extensions import deprecated  # added in 3.13

from pygame.typing import ColorLike

THECOLORS: dict[str, tuple[int, int, int, int]]

# Color confirms to the Collection ABC, since it also confirms to
# Sized, Iterable and Container ABCs
class Color(Collection[int]):
    r: int
    g: int
    b: int
    a: int
    cmy: tuple[float, float, float]
    hsva: tuple[float, float, float, float]
    hsla: tuple[float, float, float, float]
    i1i2i3: tuple[float, float, float]
    normalized: tuple[float, float, float, float]
    __hash__: None  # type: ignore
    __array_struct__: Any
    @overload
    def __init__(self, r: int, g: int, b: int, a: int = 255) -> None: ...
    @overload
    def __init__(self, rgbvalue: ColorLike) -> None: ...
    @overload
    def __getitem__(self, i: SupportsIndex) -> int: ...
    @overload
    def __getitem__(self, s: slice) -> tuple[int, ...]: ...
    def __setitem__(self, key: int, value: int) -> None: ...
    def __iter__(self) -> Iterator[int]: ...
    def __add__(self, other: Color) -> Color: ...
    def __sub__(self, other: Color) -> Color: ...
    def __mul__(self, other: Color) -> Color: ...
    def __floordiv__(self, other: Color) -> Color: ...
    def __mod__(self, other: Color) -> Color: ...
    def __int__(self) -> int: ...
    def __float__(self) -> float: ...
    def __len__(self) -> int: ...
    def __index__(self) -> int: ...
    def __invert__(self) -> Color: ...
    def __contains__(self, other: int) -> bool: ...  # type: ignore[override]
    def __getattribute__(self, attr: str) -> Union[Color, tuple[int, ...]]: ...
    def __setattr__(self, attr: str, value: Union[Color, tuple[int, ...]]) -> None: ...
    @overload
    @classmethod
    def from_cmy(cls, object: tuple[float, float, float], /) -> Color: ...
    @overload
    @classmethod
    def from_cmy(cls, c: float, m: float, y: float, /) -> Color: ...
    @overload
    @classmethod
    def from_hsva(cls, object: tuple[float, float, float, float], /) -> Color: ...
    @overload
    @classmethod
    def from_hsva(cls, h: float, s: float, v: float, a: float, /) -> Color: ...
    @overload
    @classmethod
    def from_hsla(cls, object: tuple[float, float, float, float], /) -> Color: ...
    @overload
    @classmethod
    def from_hsla(cls, h: float, s: float, l: float, a: float, /) -> Color: ...
    @overload
    @classmethod
    def from_i1i2i3(cls, object: tuple[float, float, float], /) -> Color: ...
    @overload
    @classmethod
    def from_i1i2i3(cls, i1: float, i2: float, i3: float, /) -> Color: ...
    @overload
    @classmethod
    def from_normalized(cls, object: tuple[float, float, float, float], /) -> Color: ...
    @overload
    @classmethod
    def from_normalized(cls, r: float, g: float, b: float, a: float, /) -> Color: ...
    def normalize(self) -> tuple[float, float, float, float]: ...
    def correct_gamma(self, gamma: float, /) -> Color: ...
    @deprecated("since 2.1.3. Use unpacking instead")
    def set_length(self, length: int, /) -> None: ...
    def lerp(self, color: ColorLike, amount: float) -> Color: ...
    def premul_alpha(self) -> Color: ...
    def grayscale(self) -> Color: ...
    @overload
    def update(self, r: int, g: int, b: int, a: int = 255, /) -> None: ...
    @overload
    def update(self, rgbvalue: ColorLike, /) -> None: ...



# Example file showing a basic pygame "game loop"
import pygame

# pygame setup
pygame.init()
screen = pygame.display.set_mode((1280, 720))
clock = pygame.time.Clock()
running = True

while running:
    # poll for events
    # pygame.QUIT event means the user clicked X to close your window
    for event in pygame.event.get():
        if event.type == pygame.QUIT:
            running = False

    # fill the screen with a color to wipe away anything from last frame
    screen.fill("purple")

    # RENDER YOUR GAME HERE

    # flip() the display to put your work on screen
    pygame.display.flip()

    clock.tick(60)  # limits FPS to 60

pygame.quit()



from pygame.rect import Rect
from pygame.surface import Surface
from typing import overload

from pygame.typing import ColorLike, Point, RectLike, SequenceLike

def rect(
    surface: Surface,
    color: ColorLike,
    rect: RectLike,
    width: int = 0,
    border_radius: int = -1,
    border_top_left_radius: int = -1,
    border_top_right_radius: int = -1,
    border_bottom_left_radius: int = -1,
    border_bottom_right_radius: int = -1,
) -> Rect: ...
def polygon(
    surface: Surface,
    color: ColorLike,
    points: SequenceLike[Point],
    width: int = 0,
) -> Rect: ...
def circle(
    surface: Surface,
    color: ColorLike,
    center: Point,
    radius: float,
    width: int = 0,
    draw_top_right: bool = False,
    draw_top_left: bool = False,
    draw_bottom_left: bool = False,
    draw_bottom_right: bool = False,
) -> Rect: ...
@overload
def aacircle(
    surface: Surface,
    color: ColorLike,
    center: Point,
    radius: float,
    width: int = 0,
) -> Rect: ...
@overload
def aacircle(
    surface: Surface,
    color: ColorLike,
    center: Point,
    radius: float,
    width: int = 0,
    draw_top_right: bool = False,
    draw_top_left: bool = False,
    draw_bottom_left: bool = False,
    draw_bottom_right: bool = False,
) -> Rect: ...
def ellipse(
    surface: Surface, color: ColorLike, rect: RectLike, width: int = 0
) -> Rect: ...
def arc(
    surface: Surface,
    color: ColorLike,
    rect: RectLike,
    start_angle: float,
    stop_angle: float,
    width: int = 1,
) -> Rect: ...
def line(
    surface: Surface,
    color: ColorLike,
    start_pos: Point,
    end_pos: Point,
    width: int = 1,
) -> Rect: ...
def lines(
    surface: Surface,
    color: ColorLike,
    closed: bool,
    points: SequenceLike[Point],
    width: int = 1,
) -> Rect: ...
def aaline(
    surface: Surface,
    color: ColorLike,
    start_pos: Point,
    end_pos: Point,
    width: int = 1,
) -> Rect: ...
def aalines(
    surface: Surface,
    color: ColorLike,
    closed: bool,
    points: SequenceLike[Point],
) -> Rect: ...



"""pygame.examples.window_opengl

Slightly modified version of the example at https://github.com/szabolcsdombi/zengl/blob/main/examples/pygame/hello_world.py
Modified to use Window API instead of the display module

Posted here with permission from https://github.com/szabolcsdombi given in the PGC discord server
"""

import pygame
import zengl


def main():
    window_size = (1280, 720)

    pygame.init()
    window = pygame.Window(size=window_size, opengl=True)

    ctx = zengl.context()

    image = ctx.image(window_size, "rgba8unorm", samples=4)

    pipeline = ctx.pipeline(
        vertex_shader="""
            #version 330 core

            out vec3 v_color;

            vec2 vertices[3] = vec2[](
                vec2(0.0, 0.8),
                vec2(-0.6, -0.8),
                vec2(0.6, -0.8)
            );

            vec3 colors[3] = vec3[](
                vec3(1.0, 0.0, 0.0),
                vec3(0.0, 1.0, 0.0),
                vec3(0.0, 0.0, 1.0)
            );

            void main() {
                gl_Position = vec4(vertices[gl_VertexID], 0.0, 1.0);
                v_color = colors[gl_VertexID];
            }
        """,
        fragment_shader="""
            #version 330 core

            in vec3 v_color;

            layout (location = 0) out vec4 out_color;

            void main() {
                out_color = vec4(v_color, 1.0);
                out_color.rgb = pow(out_color.rgb, vec3(1.0 / 2.2));
            }
        """,
        framebuffer=[image],
        topology="triangles",
        vertex_count=3,
    )

    clock = pygame.Clock()

    while True:
        for event in pygame.event.get():
            if event.type == pygame.QUIT:
                pygame.quit()
                quit()

        ctx.new_frame()
        image.clear()
        pipeline.render()
        image.blit()
        ctx.end_frame()

        window.flip()
        clock.tick(60)


if __name__ == "__main__":
    main()



from typing import Optional, TypedDict

from pygame._data_classes import PowerState

class _InstructionSets(TypedDict):
    ALTIVEC: bool
    MMX: bool
    SSE: bool
    SSE2: bool
    SSE3: bool
    SSE41: bool
    SSE42: bool
    AVX: bool
    AVX2: bool
    AVX512F: bool
    NEON: bool
    ARMSIMD: bool
    LSX: bool
    LASX: bool

# dict at runtime, TypedDict exists solely for the typechecking benefits
class _Locale(TypedDict):
    language: str
    country: Optional[str]

def get_cpu_instruction_sets() -> _InstructionSets: ...
def get_total_ram() -> int: ...
def get_pref_path(org: str, app: str) -> str: ...
def get_pref_locales() -> list[_Locale]: ...
def get_power_state() -> Optional[PowerState]: ...



"""
A helper script to run mypy stubtest on the stubs directory
"""

import os
import subprocess
import sys

from pathlib import Path

STUBS_BASE_DIR = Path(__file__).parent


def typing_check():
    """
    Ensure type aliases in typing.py work as expected with type checkers
    """
    mypy_version_args = [sys.executable, "-m", "mypy"]
    try:
        version = subprocess.run(
                [*mypy_version_args, "--version"], capture_output=True, check=True, text=True
            ).stdout.strip()
    except subprocess.CalledProcessError:
        print("ERROR: could not validate typing.py, make sure you have mypy installed")
        return

    mypy_args = [*mypy_version_args, "typing_sample_app.py"]
    cmd = " ".join(mypy_args)
    print(f"Using mypy invocation: `{cmd}` (version: {version})")
    prev_dir = os.getcwd()
    try:
        os.chdir(STUBS_BASE_DIR)
        returncode = subprocess.run([*mypy_args]).returncode
        if returncode != 0:
            raise RuntimeError(f"mypy process finished with unsuccessful return code {returncode}")
    finally:
        os.chdir(prev_dir)

def stubs_check():
    """
    Validate the stubs files
    """
    for stubtest in ([sys.executable, "-m", "mypy.stubtest"], ["stubtest"]):
        try:
            version = subprocess.run(
                [*stubtest, "--version"], capture_output=True, check=True, text=True
            ).stdout.strip()
        except subprocess.CalledProcessError:
            continue

        cmd = " ".join(stubtest)
        print(f"Using stubtest invocation: `{cmd}` (version: {version})")
        prev_dir = os.getcwd()
        try:
            os.chdir(STUBS_BASE_DIR)
            sys.exit(
                subprocess.run(
                    [*stubtest, "pygame", "--allowlist", "mypy_allow_list.txt"]
                ).returncode
            )
        finally:
            os.chdir(prev_dir)

    print("ERROR: Could not find a valid stubtest program.")
    print("Make sure you have mypy installed.")
    sys.exit(1)


def main():
    """
    Main entrypoint
    """
    typing_check()
    stubs_check()


if __name__ == "__main__":
    main()



from collections.abc import Callable, Iterable
from typing import Any, Optional, Union
from typing_extensions import deprecated # added in 3.13

from pygame.color import Color
from pygame.rect import Rect
from pygame.surface import Surface

from pygame.typing import ColorLike, FileLike, RectLike

def get_error() -> str: ...
def get_version(linked: bool = True) -> tuple[int, int, int]: ...
def init(cache_size: int = 64, resolution: int = 72) -> None: ...
def quit() -> None: ...
def get_init() -> bool: ...
@deprecated("Use `pygame.freetype.get_init` instead")
def was_init() -> bool: ...
def get_cache_size() -> int: ...
def get_default_resolution() -> int: ...
def set_default_resolution(resolution: int = 0, /) -> None: ...
def SysFont(
    name: Union[str, bytes, Iterable[Union[str, bytes]], None],
    size: int,
    bold: int = False,
    italic: int = False,
    constructor: Optional[Callable[[Optional[str], int, bool, bool], Font]] = None,
) -> Font: ...
def get_default_font() -> str: ...
def get_fonts() -> list[str]: ...
def match_font(
    name: Union[str, bytes, Iterable[Union[str, bytes]]],
    bold: Any = False,
    italic: Any = False,
) -> str: ...

STYLE_NORMAL: int
STYLE_UNDERLINE: int
STYLE_OBLIQUE: int
STYLE_STRONG: int
STYLE_WIDE: int
STYLE_DEFAULT: int

class Font:
    @property
    def size(self) -> Union[float, tuple[float, float]]: ...
    @size.setter
    def size(self, value: Union[float, tuple[float, float]]) -> None: ...
    @property
    def style(self) -> int: ...
    @style.setter
    def style(self, value: int) -> None: ...
    @property
    def height(self) -> int: ...
    @property
    def ascender(self) -> int: ...
    @property
    def descender(self) -> int: ...
    @property
    def name(self) -> str: ...
    @property
    def style_name(self) -> str: ...
    @property
    def path(self) -> str: ...
    @property
    def scalable(self) -> bool: ...
    @property
    def fixed_width(self) -> bool: ...
    @property
    def fixed_sizes(self) -> int: ...
    @property
    def antialiased(self) -> bool: ...
    @antialiased.setter
    def antialiased(self, value: bool) -> None: ...
    @property
    def kerning(self) -> bool: ...
    @kerning.setter
    def kerning(self, value: bool) -> None: ...
    @property
    def vertical(self) -> bool: ...
    @vertical.setter
    def vertical(self, value: bool) -> None: ...
    @property
    def pad(self) -> bool: ...
    @pad.setter
    def pad(self, value: bool) -> None: ...
    @property
    def oblique(self) -> bool: ...
    @oblique.setter
    def oblique(self, value: bool) -> None: ...
    @property
    def strong(self) -> bool: ...
    @strong.setter
    def strong(self, value: bool) -> None: ...
    @property
    def underline(self) -> bool: ...
    @underline.setter
    def underline(self, value: bool) -> None: ...
    @property
    def wide(self) -> bool: ...
    @wide.setter
    def wide(self, value: bool) -> None: ...
    @property
    def strength(self) -> float: ...
    @strength.setter
    def strength(self, value: float) -> None: ...
    @property
    def underline_adjustment(self) -> float: ...
    @underline_adjustment.setter
    def underline_adjustment(self, value: float) -> None: ...
    @property
    def ucs4(self) -> bool: ...
    @ucs4.setter
    def ucs4(self, value: bool) -> None: ...
    @property
    def use_bitmap_strikes(self) -> bool: ...
    @use_bitmap_strikes.setter
    def use_bitmap_strikes(self, value: bool) -> None: ...
    @property
    def resolution(self) -> int: ...
    @property
    def rotation(self) -> int: ...
    @rotation.setter
    def rotation(self, value: int) -> None: ...
    @property
    def fgcolor(self) -> Color: ...
    @fgcolor.setter
    def fgcolor(self, value: ColorLike) -> None: ...
    @property
    def bgcolor(self) -> Color: ...
    @bgcolor.setter
    def bgcolor(self, value: ColorLike) -> None: ...
    @property
    def origin(self) -> bool: ...
    @origin.setter
    def origin(self, value: bool) -> None: ...
    def __init__(
        self,
        file: Optional[FileLike],
        size: float = 0,
        font_index: int = 0,
        resolution: int = 0,
        ucs4: int = False,
    ) -> None: ...
    def get_rect(
        self,
        text: Optional[str],
        style: int = STYLE_DEFAULT,
        rotation: int = 0,
        size: float = 0,
    ) -> Rect: ...
    def get_metrics(
        self, text: str, size: float = 0
    ) -> list[tuple[int, int, int, int, float, float]]: ...
    def get_sized_ascender(self, size: float, /) -> int: ...
    def get_sized_descender(self, size: float, /) -> int: ...
    def get_sized_height(self, size: float, /) -> int: ...
    def get_sized_glyph_height(self, size: float, /) -> int: ...
    def get_sizes(self) -> list[tuple[int, int, int, float, float]]: ...
    def render(
        self,
        text: Optional[str],
        fgcolor: Optional[ColorLike] = None,
        bgcolor: Optional[ColorLike] = None,
        style: int = STYLE_DEFAULT,
        rotation: int = 0,
        size: float = 0,
    ) -> tuple[Surface, Rect]: ...
    def render_to(
        self,
        surf: Surface,
        dest: RectLike,
        text: Optional[str],
        fgcolor: Optional[ColorLike] = None,
        bgcolor: Optional[ColorLike] = None,
        style: int = STYLE_DEFAULT,
        rotation: int = 0,
        size: float = 0,
    ) -> Rect: ...
    def render_raw(
        self,
        text: Optional[str],
        style: int = STYLE_DEFAULT,
        rotation: int = 0,
        size: float = 0,
        invert: bool = False,
    ) -> tuple[bytes, tuple[int, int]]: ...
    def render_raw_to(
        self,
        array: Any,
        text: Optional[str],
        dest: Optional[RectLike] = None,
        style: int = STYLE_DEFAULT,
        rotation: int = 0,
        size: float = 0,
        invert: bool = False,
    ) -> Rect: ...



from typing import Any, Optional, Union, final
from typing_extensions import deprecated  # added in 3.13

from pygame.typing import SequenceLike

class _GenericEvent:
    # Just exists to avoid duplication of data for Event
    # and (deprecated) EventType

    @property
    def type(self) -> int: ...
    __dict__: dict[str, Any]
    __hash__: None  # type: ignore
    def __init__(
        self, type: int, dict: dict[str, Any] = ..., **kwargs: Any
    ) -> None: ...
    def __getattribute__(self, name: str) -> Any: ...
    def __setattr__(self, name: str, value: Any) -> None: ...
    def __delattr__(self, name: str) -> None: ...
    def __bool__(self) -> bool: ...

    # this is at the bottom because mypy complains if this declaration comes
    # before any uses of the dict[] typehinting because of the same naming
    @property
    def dict(self) -> dict[str, Any]: ...

@final
class Event(_GenericEvent):
    pass

@final
@deprecated("Use `Event` instead (EventType is an old alias)")
class EventType(_GenericEvent):
    pass

_EventTypes = Union[int, SequenceLike[int]]

def pump() -> None: ...
def get(
    eventtype: Optional[_EventTypes] = None,
    pump: bool = True,
    exclude: Optional[_EventTypes] = None,
) -> list[Event]: ...
def poll() -> Event: ...
def wait(timeout: int = 0) -> Event: ...
def peek(eventtype: Optional[_EventTypes] = None, pump: bool = True) -> bool: ...
def clear(eventtype: Optional[_EventTypes] = None, pump: bool = True) -> None: ...
def event_name(type: int, /) -> str: ...
def set_blocked(type: Optional[_EventTypes], /) -> None: ...
def set_allowed(type: Optional[_EventTypes], /) -> None: ...
def get_blocked(type: _EventTypes, /) -> bool: ...
def set_grab(grab: bool, /) -> None: ...
def get_grab() -> bool: ...
def post(event: Event, /) -> bool: ...
def custom_type() -> int: ...



"""Config on Emscripten SDK is almost like Unix"""

import logging
import os
import sys
from glob import glob

from distutils.sysconfig import get_python_inc

configcommand = os.environ.get(
    "SDL_CONFIG",
    "sdl2-config",
)
configcommand = configcommand + " --version --cflags --libs"
localbase = os.environ.get("LOCALBASE", "")
if os.environ.get("PYGAME_EXTRA_BASE"):
    extrabases = os.environ["PYGAME_EXTRA_BASE"].split(":")
else:
    extrabases = []

EMSDK = os.environ.get("EMSDK", None)
is_wasm = EMSDK is not None
if not is_wasm:
    print("EMSDK not found")
    raise SystemExit(1)

# EMCC_CFLAGS="-s USE_SDL=2 is required to prevent '-iwithsysroot/include/SDL'
# which is SDL1 from ./emscripten/tools/ports/__init__.py
# but CI is not expected to have that problem and that would trigger useless
# rebuild

EMCC_CFLAGS = os.environ.get("EMCC_CFLAGS", "")

# user build, make sure ports are pulled in.
if os.environ.get("SDK_VERSION", None) is None:
    EMCC_CFLAGS += " -sUSE_SDL=2"
else:
    # make sure CI only pick SDK components.
    SDKROOT = os.environ.get("SDKROOT", "/opt/python-wasm-sdk")
    emcc_cflags = [
        # only static build is supported at the moment
        "-DBUILD_STATIC",
        # for SDL2_image
        f"-I{SDKROOT}/devices/emsdk/usr/include/SDL2",
        # ftbuild.h
        f"-I{SDKROOT}/emsdk/upstream/emscripten/cache/sysroot/include/freetype2",
        # avoid some cython generated warnings.
        f"-Wno-unreachable-code-fallthrough {EMCC_CFLAGS}",
    ]
    EMCC_CFLAGS = " ".join(emcc_cflags)
os.environ["EMCC_CFLAGS"] = EMCC_CFLAGS.strip()


CC = os.environ.get("CC", "emcc")
os.environ["CC"] = CC.strip()


class DependencyProg:
    def __init__(
        self, name, envname, exename, minver, defaultlibs, version_flag="--version"
    ):
        self.name = name
        command = os.environ.get(envname, exename)
        self.lib_dir = ""
        self.inc_dir = ""
        self.libs = []
        self.cflags = ""
        try:
            # freetype-config for freetype2 version 2.3.7 on Debian lenny
            # does not recognize multiple command line options. So execute
            # 'command' separately for each option.
            config = (
                os.popen(command + " " + version_flag).readlines()
                + os.popen(command + " --cflags").readlines()
                + os.popen(command + " --libs").readlines()
            )
            flags = " ".join(config[1:]).split()

            # remove this GNU_SOURCE if there... since python has it already,
            #   it causes a warning.
            if "-D_GNU_SOURCE=1" in flags:
                flags.remove("-D_GNU_SOURCE=1")
            self.ver = config[0].strip()
            if minver and self.ver < minver:
                err = (
                    f"WARNING: requires {self.name} version {self.ver} ({minver} found)"
                )
                raise ValueError(err)
            self.found = 1
            self.cflags = ""
            for f in flags:
                if f[:2] in ("-l", "-D", "-I", "-L"):
                    self.cflags += f + " "
                elif f[:3] == "-Wl":
                    self.cflags += "-Xlinker " + f + " "

            if self.name == "SDL":
                inc = f"-I{EMSDK}/upstream/emscripten/cache/sysroot/include/SDL2 "
                inc += f"-I{EMSDK}/upstream/emscripten/cache/sysroot/include/freetype2/freetype "
                self.cflags = inc + " " + self.cflags

            if self.name == "FREETYPE":
                inc = f"-I{EMSDK}/upstream/emscripten/cache/sysroot/include/freetype2/freetype "
                self.cflags = inc + " " + self.cflags

        except (ValueError, TypeError):
            print(f'WARNING: "{command}" failed!')
            self.found = 0
            self.ver = "0"
            self.libs = defaultlibs

    def configure(self, incdirs, libdir):
        if self.found:
            print(self.name + "        "[len(self.name) :] + ": found " + self.ver)
            self.found = 1
        else:
            print(self.name + "        "[len(self.name) :] + ": not found")


class Dependency:
    def __init__(self, name, checkhead, checklib, libs):
        self.name = name
        self.inc_dir = None
        self.lib_dir = None
        self.libs = libs
        self.found = 0
        self.checklib = checklib
        self.checkhead = checkhead
        self.cflags = ""

    def configure(self, incdirs, libdirs):
        incname = self.checkhead
        libnames = self.checklib, self.name.lower()

        if incname:
            for dir in incdirs:
                path = os.path.join(dir, incname)
                if os.path.isfile(path):
                    self.inc_dir = dir

        for dir in libdirs:
            for name in libnames:
                path = os.path.join(dir, name)
                if any(map(os.path.isfile, glob(path + "*"))):
                    self.lib_dir = dir

        if (incname and self.lib_dir and self.inc_dir) or (
            not incname and self.lib_dir
        ):
            print(self.name + "        "[len(self.name) :] + ": found")
            self.found = 1
        else:

            if self.name in ["FONT", "IMAGE", "MIXER", "FREETYPE"]:
                self.found = 1
                print(
                    self.name
                    + "        "[len(self.name) :]
                    + ": FORCED (via emsdk builtins)"
                )
                return
            print(self.name + "        "[len(self.name) :] + ": not found")
            print(self.name, self.checkhead, self.checklib, incdirs, libdirs)


class DependencyPython:
    def __init__(self, name, module, header):
        self.name = name
        self.lib_dir = ""
        self.inc_dir = ""
        self.libs = []
        self.cflags = ""
        self.found = 0
        self.ver = "0"
        self.module = module
        self.header = header

    def configure(self, incdirs, libdirs):
        self.found = 1
        if self.module:
            try:
                self.ver = __import__(self.module).__version__
            except ImportError:
                self.found = 0
        if self.found and self.header:
            fullpath = os.path.join(get_python_inc(0), self.header)
            if not os.path.isfile(fullpath):
                self.found = 0
            else:
                self.inc_dir = os.path.split(fullpath)[0]
        if self.found:
            print(self.name + "        "[len(self.name) :] + ": found", self.ver)
        else:
            print(self.name + "        "[len(self.name) :] + ": not found")


sdl_lib_name = "SDL"


def main(auto_config=False):
    global origincdirs, origlibdirs

    # these get prefixes with $EMSDK
    origincdirs = ["/include"]
    origlibdirs = []

    print("\nHunting dependencies...")

    DEPS = [
        DependencyProg("SDL", "SDL_CONFIG", "sdl2-config", "2.0", ["sdl"]),
        Dependency("FONT", "SDL_ttf.h", "libSDL2_ttf.so", ["SDL2_ttf"]),
        Dependency("IMAGE", "SDL_image.h", "libSDL2_image.a", ["SDL2_image"]),
        Dependency("MIXER", "SDL_mixer.h", "libSDL2_mixer.a", ["SDL2_mixer"]),
        Dependency("FREETYPE", "ft2build.h", "libfreetype.a", []),
        # Dependency('GFX', 'SDL_gfxPrimitives.h', 'libSDL2_gfx.a', ['SDL2_gfx']),
    ]
    DEPS.extend(
        [
            # Dependency('SCRAP', '', 'libX11', ['X11']),
            # Dependency('GFX', 'SDL_gfxPrimitives.h', 'libSDL_gfx.a', ['SDL_gfx']),
        ]
    )

    if not DEPS[0].found:
        raise RuntimeError(
            'Unable to run "sdl-config". Please make sure a development version of SDL is installed.'
        )

    incdirs = []
    libdirs = []

    incdirs += [os.environ.get("PREFIX", "") + d for d in origincdirs]
    libdirs += [os.environ.get("PREFIX", "") + d for d in origlibdirs]

    incdirs += [EMSDK + "/upstream/emscripten/cache/sysroot" + d for d in origincdirs]

    for extrabase in extrabases:
        incdirs += [extrabase + d for d in origincdirs]
        libdirs += [extrabase + d for d in origlibdirs]

    if localbase:
        incdirs = [localbase + d for d in origincdirs]
        libdirs = [localbase + d for d in origlibdirs]

    for arg in DEPS[0].cflags.split():
        if arg[:2] == "-I":
            incdirs.append(arg[2:])
        elif arg[:2] == "-L":
            libdirs.append(arg[2:])
    for d in DEPS:
        d.configure(incdirs, libdirs)

    for d in DEPS[1:]:
        if not d.found:
            if "-auto" not in sys.argv:
                logging.warning(
                    "Some pygame dependencies were not found. "
                    "Pygame can still compile and install, but games that "
                    "depend on those missing dependencies will not run. "
                    "Use -auto to continue building without all dependencies. "
                )
                raise SystemExit("Missing dependencies")
            break
    return DEPS


if __name__ == "__main__":
    print("This is the configuration subscript for Emscripten.")
    print('Please run "config.py" for full configuration.')



"""Sample app run by mypy to ensure typing.py aliases work as expected"""

from pygame import typing
import pygame
import pathlib


# validate SequenceLike
class MySequence:
    def __getitem__(self, index: int) -> str:
        if index > 20:
            raise IndexError()
        if index % 2 == 0:
            return "a"
        return "bc"

    def __len__(self) -> int:
        return 20

def validator_SequenceLike(sequence: typing.SequenceLike) -> int:
    return 0

def validator_SequenceLikeTypes(
    sequence_float: typing.SequenceLike[float],
    sequence_int: typing.SequenceLike[int],
    sequence_str: typing.SequenceLike[str],
    sequence_sequence: typing.SequenceLike[typing.SequenceLike],
) -> int:
    return 0

# must pass
validator_SequenceLike(MySequence())
validator_SequenceLike([0, 1, 2, 3])
validator_SequenceLike((0, 1, 2, 3))
validator_SequenceLike(pygame.Rect(-10, 10, 40, 40))
validator_SequenceLike(pygame.Vector2())
validator_SequenceLike("1234567890")

validator_SequenceLikeTypes(
    (-1.5, -0.5, 0, 0.5, 2.5, 10),
    (-2, -1, 0, 1, 2, 3),
    "abcdefghijklmnopqrstuvwxyz",
    [(0.5, 1.5), (-1, 1), "123", [(), (), ()]],
)


# validate _PathLike
class MyPath:
    def __fspath__(self) -> str:
        return "file.py"

def validator_PathLike(path: typing._PathLike) -> int:
    return 0

# must pass
validator_PathLike("file.py")
validator_PathLike(b"file.py")
validator_PathLike(pathlib.Path("file.py"))
validator_PathLike(MyPath())


# validate Point, IntPoint
def validator_Point(point: typing.Point) -> int:
    return 0

def validator_IntPoint(point: typing.IntPoint) -> int:
    return 0

# must pass
validator_Point((1, 2))
validator_Point([3, -4])
validator_Point((5, -6.5))
validator_Point((-6.7, 8.9))
validator_Point(pygame.Vector2())

validator_IntPoint((3, 4))
validator_IntPoint([-4, -3])


def validator_ColorLike(color: typing.ColorLike) -> int:
    return 0

# must pass
validator_ColorLike("green")
validator_ColorLike(1)
validator_ColorLike((255, 255, 255, 30))
validator_ColorLike(pygame.Color(100, 100, 100, 100))


# validate RectLike
class MyObject1:
    def __init__(self):
        self.rect = pygame.Rect(10, 10, 20, 20)

class MyObject2:
    def __init__(self):
        self.rect = lambda: pygame.Rect(5, 5, 10, 10)

class MyObject3:
    def rect(self) -> pygame.Rect:
        return pygame.Rect(15, 15, 30, 30)

def validator_RectLike(rect: typing.RectLike) -> int:
    return 0

# must pass
validator_RectLike((10, 10, 10, 10))
validator_RectLike(((5, 5), (30, 30)))
validator_RectLike(([3, 3.2], pygame.Vector2()))
validator_RectLike(pygame.Rect(1, 2, 3, 4))
validator_RectLike(MyObject1())
validator_RectLike(MyObject2())
validator_RectLike(MyObject3())



#!/usr/bin/env python
"""pygame.examples.scrap_clipboard

Demonstrates the clipboard capabilities of pygame.

Copy/paste!


Keyboard Controls
-----------------
    v - View the current clipboard data.
    c - Copy some text into the clipboard.

"""

import pygame


def main():
    pygame.init()
    pygame.display.set_caption("Clipboard Example")
    width, height = (960, 540)
    screen = pygame.display.set_mode((width, height))
    clock = pygame.Clock()
    font = pygame.Font(None, 30)

    clipboard_text = ""
    running = True

    while running:
        screen.fill("black")

        instruction = "Keyboard Controls:\nV - View the current clipboard data.\nC - Copy some text into the clipboard.\nEscape - Quit"
        text = font.render(instruction, True, "white")
        screen.blit(text, (0, 0))

        text = font.render(
            f"Text on the clipboard:\n{clipboard_text}", True, "white", None, width - 20
        )
        screen.blit(text, (0, 100))

        for event in pygame.event.get():
            if event.type == pygame.QUIT:
                running = False

            elif event.type == pygame.KEYDOWN:
                if event.key == pygame.K_v:
                    # Look for any text data in the clipboard.
                    print("Looking for text in the clipboard.")
                    if pygame.scrap.has_text():
                        print("Text found in the clipboard.")
                        clipboard_text = pygame.scrap.get_text()
                    else:
                        print("No text in the clipboard.")

                elif event.key == pygame.K_c:
                    # put some text into the clipboard.
                    print("Putting text into the clipboard.")

                    pygame.scrap.put_text(
                        "Hello World! This is some text from the pygame scrap example.",
                    )

                elif event.key == pygame.K_ESCAPE:
                    running = False

            pygame.display.flip()
            clock.tick(60)
    pygame.quit()


if __name__ == "__main__":
    main()



from typing import Literal, overload
from typing_extensions import deprecated # added in 3.13

from pygame.cursors import Cursor
from pygame.surface import Surface

from pygame.typing import Point, SequenceLike, IntPoint

@overload
def get_pressed(
    num_buttons: Literal[3] = 3,
    desktop: bool = False) -> tuple[bool, bool, bool]: ...
@overload
def get_pressed(
    num_buttons: Literal[5],
    desktop: bool = False) -> tuple[bool, bool, bool, bool, bool]: ...
def get_just_pressed() -> tuple[bool, bool, bool, bool, bool]: ...
def get_just_released() -> tuple[bool, bool, bool, bool, bool]: ...
def get_pos(desktop: bool = False) -> tuple[int, int]: ...
def get_rel() -> tuple[int, int]: ...
@overload
def set_pos(pos: Point, /) -> None: ...
@overload
def set_pos(x: float, y: float, /) -> None: ...
def set_visible(value: bool, /) -> int: ...
def get_visible() -> bool: ...
def get_focused() -> bool: ...
@overload
def set_cursor(cursor: Cursor) -> None: ...
@overload
def set_cursor(constant: int) -> None: ...
@overload
def set_cursor(
    size: IntPoint,
    hotspot: IntPoint,
    xormasks: SequenceLike[int],
    andmasks: SequenceLike[int],
) -> None: ...
@overload
def set_cursor(hotspot: IntPoint, surface: Surface) -> None: ...
def get_cursor() -> Cursor: ...
@deprecated("since 2.2.0. Use `pygame.mouse.set_cursor` instead")
def set_system_cursor(cursor: int, /) -> None: ...
def get_relative_mode() -> bool: ...
def set_relative_mode(enable: bool, /) -> None: ...



#!/usr/bin/env python
"""pygame.examples.dropfile

Drag and drop an image on here.

Uses these events:

* DROPBEGIN
* DROPCOMPLETE
* DROPTEXT
* DROPFILE
"""

import pygame

pygame.init()


def main():
    running = True
    screen_size = (640, 480)
    surf = pygame.display.set_mode(screen_size)
    font = pygame.font.SysFont("Arial", 24)
    font.align = pygame.FONT_CENTER
    clock = pygame.Clock()

    spr_file_text = font.render("Feed me some file or image!", True, (255, 255, 255))
    spr_file_text_rect = spr_file_text.get_rect()
    spr_file_text_rect.center = surf.get_rect().center

    spr_file_image = None
    spr_file_image_rect = None

    while running:
        for event in pygame.event.get():
            if event.type == pygame.QUIT:
                running = False
            elif event.type == pygame.DROPBEGIN:
                print(event)
                print("File drop begin!")
            elif event.type == pygame.DROPCOMPLETE:
                print(event)
                print("File drop complete!")
            elif event.type == pygame.DROPTEXT:
                print(event)
                spr_file_text = font.render(
                    event.text, True, (255, 255, 255), wraplength=screen_size[0] - 10
                )
                spr_file_text_rect = spr_file_text.get_rect()
                spr_file_text_rect.center = surf.get_rect().center
            elif event.type == pygame.DROPFILE:
                print(event)
                spr_file_text = font.render(
                    event.file, True, (255, 255, 255), None, screen_size[0] - 10
                )
                spr_file_text_rect = spr_file_text.get_rect()
                spr_file_text_rect.center = surf.get_rect().center

                # Try to open the file if it's an image
                filetype = event.file[-3:]
                if filetype in ["png", "bmp", "jpg"]:
                    spr_file_image = pygame.image.load(event.file).convert()
                    spr_file_image.set_alpha(127)
                    spr_file_image_rect = spr_file_image.get_rect()
                    spr_file_image_rect.center = surf.get_rect().center

        surf.fill((0, 0, 0))
        surf.blit(spr_file_text, spr_file_text_rect)
        if spr_file_image:
            surf.blit(spr_file_image, spr_file_image_rect)

        pygame.display.flip()
        clock.tick(30)

    pygame.quit()


if __name__ == "__main__":
    main()



from collections.abc import Iterable
from typing import Optional, Union, overload, Literal
from typing_extensions import deprecated # added in 3.13

from pygame.constants import FULLSCREEN
from pygame.surface import Surface

from pygame._sdl2 import Window

from pygame.typing import (
    ColorLike,
    Point,
    IntPoint,
    RectLike,
    SequenceLike,
)

class _VidInfo:
    hw: int
    wm: int
    video_mem: int
    bitsize: int
    bytesize: int
    masks: tuple[int, int, int, int]
    shifts: tuple[int, int, int, int]
    losses: tuple[int, int, int, int]
    blit_hw: int
    blit_hw_CC: int
    blit_hw_A: int
    blit_sw: int
    blit_sw_CC: int
    blit_sw_A: int
    current_h: int
    current_w: int
    pixel_format: str

def init() -> None: ...
def quit() -> None: ...
def get_init() -> bool: ...
def set_mode(
    size: Point = (0, 0),
    flags: int = 0,
    depth: int = 0,
    display: int = 0,
    vsync: int = 0,
) -> Surface: ...
def get_surface() -> Optional[Surface]: ...
def flip() -> None: ...
@overload
def update() -> None: ...
@overload
def update(
    rectangle: Optional[Union[RectLike, Iterable[Optional[RectLike]]]], /
) -> None: ...
@overload
def update(x: float, y: float, w: float, h: float, /) -> None: ...
@overload
def update(xy: Point, wh: Point, /) -> None: ...
def get_driver() -> str: ...
def Info() -> _VidInfo: ...
def get_wm_info() -> dict[str, int]: ...
def list_modes(
    depth: int = 0,
    flags: int = FULLSCREEN,
    display: int = 0,
) -> list[tuple[int, int]]: ...
def mode_ok(
    size: IntPoint,
    flags: int = 0,
    depth: int = 0,
    display: int = 0,
) -> int: ...
def gl_get_attribute(flag: int, /) -> int: ...
def gl_set_attribute(flag: int, value: int, /) -> None: ...
def get_active() -> bool: ...
def iconify() -> bool: ...
def toggle_fullscreen() -> int: ...
@deprecated("since 2.1.4. Removed in SDL3")
def set_gamma(red: float, green: float = ..., blue: float = ..., /) -> int: ...
@deprecated("since 2.1.4. Removed in SDL3")
def set_gamma_ramp(
    red: SequenceLike[int], green: SequenceLike[int], blue: SequenceLike[int], /
) -> int: ...
def set_icon(surface: Surface, /) -> None: ...
def set_caption(title: str, icontitle: Optional[str] = None, /) -> None: ...
def get_caption() -> tuple[str, str]: ...
def set_palette(palette: SequenceLike[ColorLike], /) -> None: ...
def get_num_displays() -> int: ...
def get_window_size() -> tuple[int, int]: ...
def get_window_position() -> tuple[int, int]:...
def set_window_position(position: Point) -> None:...
def get_allow_screensaver() -> bool: ...
def set_allow_screensaver(value: bool = True) -> None: ...
def get_desktop_sizes() -> list[tuple[int, int]]: ...
def is_fullscreen() -> bool: ...
def is_vsync() -> bool: ...
def get_current_refresh_rate() -> int: ...
def get_desktop_refresh_rates() -> list[int]: ...
def message_box(
    title: str,
    message: Optional[str] = None,
    message_type: Literal["info", "warn", "error"] = "info",
    parent_window: Optional[Window] = None,
    buttons: SequenceLike[str] = ("OK",),
    return_button: int = 0,
    escape_button: Optional[int] = None,
) -> int: ...



import unittest
import os

import pygame


class SystemModuleTest(unittest.TestCase):
    def test_get_cpu_instruction_sets(self):
        instruction_sets = pygame.system.get_cpu_instruction_sets()

        for name in instruction_sets:
            self.assertIsInstance(name, str)
            self.assertIsInstance(instruction_sets[name], bool)

    def test_get_total_ram(self):
        ram = pygame.system.get_total_ram()

        self.assertIsInstance(ram, int)

    def test_get_pref_path(self):
        get_pref_path = pygame.system.get_pref_path

        """ test the argument validation """
        # wrong arg count
        self.assertRaises(TypeError, get_pref_path, "one arg")
        # wrong arg types
        self.assertRaises(TypeError, get_pref_path, 0, 1)
        # wrong kwargs
        self.assertRaises(TypeError, get_pref_path, norg="test", napp="app")
        # not enough info
        self.assertRaises(TypeError, get_pref_path, "testorg", org="testorg")

        org = "pygame test organization"
        app = "the best app"

        # gets the path, creates the folder
        path = get_pref_path(org, app)
        try:  # try removing the folder, it should work fine
            os.rmdir(path)
        except FileNotFoundError:  # if the folder isn't found
            raise FileNotFoundError("pygame.system.get_pref_path folder not created")
        except OSError:  # if the dir isn't empty (shouldn't happen)
            raise OSError("pygame.system.get_pref_path folder already occupied")

        # gets the path, creates the folder, uses kwargs
        path = get_pref_path(org=org, app=app)
        try:  # try removing the folder, it should work fine
            os.rmdir(path)
        except FileNotFoundError:  # if the folder isn't found
            raise FileNotFoundError("pygame.system.get_pref_path folder not created")
        except OSError:  # if the dir isn't empty (shouldn't happen)
            raise OSError("pygame.system.get_pref_path folder already occupied")

    def test_get_pref_locales(self):
        locales = pygame.system.get_pref_locales()

        # check type of return first
        self.assertIsInstance(locales, list)
        for lc in locales:
            self.assertIsInstance(lc, dict)
            lang = lc["language"]
            self.assertIsInstance(lang, str)

            # length of language code should be greater than 1
            self.assertTrue(len(lang) > 1)

            country = lc["country"]
            if country is not None:
                # country field is optional, but when defined it should be a
                # string
                self.assertIsInstance(country, str)

                # length of country code should be greater than 1
                self.assertTrue(len(country) > 1)

        # passing args should raise error
        for arg in (None, 1, "hello"):
            self.assertRaises(TypeError, pygame.system.get_pref_locales, arg)

    def test_get_power_state(self):
        power_state = pygame.system.get_power_state()
        self.assertIsInstance(power_state, (type(None), pygame.system.PowerState))

        expected_types = {
            "battery_seconds": (type(None), int),
            "battery_percent": (type(None), int),
            "on_battery": bool,
            "no_battery": bool,
            "charging": bool,
            "charged": bool,
            "plugged_in": bool,
            "has_battery": bool,
        }

        if power_state is not None:
            for attr_name in expected_types:
                self.assertTrue(hasattr(power_state, attr_name))
                self.assertIsInstance(
                    getattr(power_state, attr_name), expected_types[attr_name]
                )

            self.assertTrue(power_state.plugged_in == (not power_state.on_battery))
            self.assertTrue(
                power_state.has_battery
                == (power_state.on_battery or not power_state.no_battery)
            )

            # There should be only one `True`
            self.assertEqual(
                sum(
                    [
                        power_state.on_battery,
                        power_state.no_battery,
                        power_state.charged,
                        power_state.charging,
                    ]
                ),
                1,
            )


if __name__ == "__main__":
    unittest.main()



# Example file showing a circle moving on screen
import pygame

# pygame setup
pygame.init()
screen = pygame.display.set_mode((1280, 720))
clock = pygame.time.Clock()
running = True
dt = 0

player_pos = pygame.Vector2(screen.get_width() / 2, screen.get_height() / 2)

while running:
    # poll for events
    # pygame.QUIT event means the user clicked X to close your window
    for event in pygame.event.get():
        if event.type == pygame.QUIT:
            running = False

    # fill the screen with a color to wipe away anything from last frame
    screen.fill("purple")

    pygame.draw.circle(screen, "red", player_pos, 40)

    keys = pygame.key.get_pressed()
    if keys[pygame.K_w]:
        player_pos.y -= 300 * dt
    if keys[pygame.K_s]:
        player_pos.y += 300 * dt
    if keys[pygame.K_a]:
        player_pos.x -= 300 * dt
    if keys[pygame.K_d]:
        player_pos.x += 300 * dt

    # flip() the display to put your work on screen
    pygame.display.flip()

    # limits FPS to 60
    # dt is delta time in seconds since last frame, used for framerate-
    # independent physics.
    dt = clock.tick(60) / 1000

pygame.quit()



#!/usr/bin/env python
"""pygame.examples.stars

We are all in the gutter,
but some of us are looking at the stars.
                                        -- Oscar Wilde

A simple starfield example. Note you can move the center of
the starfield by leftclicking in the window. This example shows
the basics of creating a window, simple pixel plotting, and event management.
"""

import random

import pygame


class Particle:
    def __init__(
        self,
        pos: list[int],
        vel: pygame.Vector2,
    ):
        """
        Parameters:
            pos: Position of the particle
            vel: How far does the particle move (x, y) every frame
        """

        self.pos = pos
        self.vel = vel
        self.color_list = [
            (255, 210, 125),
            (255, 163, 113),
            (166, 168, 255),
            (255, 250, 134),
            (168, 123, 255),
        ]

    def draw(self, display: pygame.Surface):
        """
        Draws the particle on a pygame.Surface
        Parameters:
            display: The surface the particle is drawn on
        """

        pygame.draw.line(display, random.choice(self.color_list), self.pos, self.pos)

    def update(self):
        """
        Moves the particle
        """

        self.pos += self.vel


def create_particle(particle_list: list[Particle], pos: pygame.Vector2):
    """
    Creates a new particle
    Parameters:
        particle_list: List of existing particles
        pos: The coordinates the new particle will be spawned on
    """

    particle_list.append(
        Particle(
            pos=pos.copy(),
            vel=pygame.Vector2(random.uniform(-5, 5), random.uniform(-5, 5)),
        )
    )


def update_particles(particle_list: list[Particle], screen_rect: pygame.Rect):
    """
    Updates the particles
    Parameters:
        particle_list: List of existing particles
        screen_rect: A pygame.Rect that represents the screen
                (used to determine whether a particle is visible on the screen)
    """

    for particle in particle_list:
        if not screen_rect.collidepoint(particle.pos):
            particle_list.remove(particle)

        particle.update()


def draw_particles(particle_list: list[Particle], display: pygame.Surface):
    """
    Draws the particles
    Parameters:
        particle_list: List of existing particles
        display: The surface the particle is drawn on
    """
    for particle in particle_list:
        particle.draw(display)


def main():
    """
    Contains the game variables and loop
    """
    screen = pygame.display.set_mode((600, 500))
    clock = pygame.Clock()
    pygame.display.set_caption("Pygame Stars")
    particles = []
    # how many particles to spawn every frame
    particle_density = 10

    screen_rect = screen.get_rect()
    spawn_point = pygame.Vector2(screen_rect.center)

    running = True
    while running:
        for event in pygame.event.get():
            if event.type == pygame.QUIT:
                running = False
            if event.type == pygame.MOUSEBUTTONDOWN:
                spawn_point = pygame.Vector2(event.pos)

        screen.fill((20, 20, 40))
        for _ in range(particle_density):
            create_particle(particles, spawn_point)
        update_particles(particles, screen_rect)
        draw_particles(particles, screen)

        pygame.display.flip()
        clock.tick(60)

    pygame.quit()


if __name__ == "__main__":
    main()



from sphinx.domains.changeset import versionlabels, VersionChange
from sphinx.locale import _  # just to suppress warnings

try:
    from sphinx.domains.changeset import versionlabel_classes
except ImportError:
    # versionlabel_classes doesn't exist in old Sphinx versions.
    UPDATE_VERIONLABEL_CLASSES = False
else:
    UPDATE_VERIONLABEL_CLASSES = True


labels = ("versionadded", "versionaddedold", "versionchanged", "versionchangedold", "deprecated", "deprecatedold", "versionextended", "versionextendedold")


def set_version_formats(app, config):
    for label in labels:
        versionlabels[label] = _(getattr(config, f"{label}_format"))


def setup(app):
    app.add_directive("versionextended", VersionChange)
    app.add_directive("versionaddedold", VersionChange)
    app.add_directive("versionchangedold", VersionChange)
    app.add_directive("deprecatedold", VersionChange)
    app.add_directive("versionextendedold", VersionChange)

    versionlabels["versionextended"] = "Extended in pygame-ce %s"
    versionlabels["versionaddedold"] = "New in pygame %s"
    versionlabels["versionchangedold"] = "Changed in pygame %s"
    versionlabels["deprecatedold"] = "Deprecated since pygame %s"
    versionlabels["versionextendedold"] = "Extended in pygame %s"


    if UPDATE_VERIONLABEL_CLASSES:
        versionlabel_classes["versionextended"] = "extended"
        versionlabel_classes["versionaddedold"] = "added"
        versionlabel_classes["versionchangedold"] = "changed"
        versionlabel_classes["deprecatedold"] = "deprecated"
        versionlabel_classes["versionextendedold"] = "extended"

    for label in labels:
        app.add_config_value(
            f"{label}_format", str(versionlabels[label]), "env"
        )

    app.connect("config-inited", set_version_formats)



import sys
from typing import Any, Union, overload

from pygame.surface import Surface
from pygame.color import Color
from pygame.typing import SequenceLike

# 'ellipsis' existed in typeshed pre 3.10, now we use EllipsisType which is
# the modern standard library equivalent.
if sys.version_info >= (3, 10):
    from types import EllipsisType
else:
    EllipsisType = ellipsis

_ColorLike = int | Color | tuple[int, int, int] | tuple[int, int, int, int]

class PixelArray:
    surface: Surface
    itemsize: int
    ndim: int
    shape: tuple[int, ...]
    strides: tuple[int, ...]
    # possibly going to be deprecated/removed soon, in which case these
    # typestubs must be removed too
    __array_interface__: dict[str, Any]
    __array_struct__: Any
    def __init__(self, surface: Surface) -> None: ...
    def __enter__(self) -> PixelArray: ...
    def __exit__(self, *args, **kwargs) -> None: ...
    # if indexing into a 2D PixelArray, a 1D PixelArray will be returned
    # if indexing into a 1D PixelArray, an int will be returned
    @overload
    def __getitem__(self, index: int) -> Union[PixelArray, int]: ...
    # complicated, but I'm pretty sure this is guaranteed to return a PixelArray or None
    # will only return None if the slice start and end are the same
    @overload
    def __getitem__(self, index_range: slice) -> Union[PixelArray, None]: ...
    # only valid for a 2D PixelArray
    @overload
    def __getitem__(self, indices: tuple[int, int]) -> int: ...
    # returns self
    @overload
    def __getitem__(self, ell: EllipsisType) -> PixelArray: ...
    def make_surface(self) -> Surface: ...
    def replace(
        self,
        color: _ColorLike,
        repcolor: _ColorLike,
        distance: float = 0,
        weights: SequenceLike[float] = (0.299, 0.587, 0.114),
    ) -> None: ...
    def extract(
        self,
        color: _ColorLike,
        distance: float = 0,
        weights: SequenceLike[float] = (0.299, 0.587, 0.114),
    ) -> PixelArray: ...
    def compare(
        self,
        array: PixelArray,
        distance: float = 0,
        weights: SequenceLike[float] = (0.299, 0.587, 0.114),
    ) -> PixelArray: ...
    def transpose(self) -> PixelArray: ...
    def close(self) -> PixelArray: ...



#!/usr/bin/env python
"""pygame.examples.scaletest

Shows an interactive image scaler.

"""

import sys
import time
import pygame


def main(imagefile, convert_alpha=False, run_speed_test=False):
    """show an interactive image scaler

    Args:
        imagefile - name of source image (required)
        convert_alpha - use convert_alpha() on the surf (default False)
        run_speed_test - (default False)
    """

    # initialize display
    pygame.display.init()
    # load background image
    background = pygame.image.load(imagefile)

    if run_speed_test:
        if convert_alpha:
            # convert_alpha() requires the display mode to be set
            pygame.display.set_mode((1, 1))
            background = background.convert_alpha()

        speed_test(background)
        return

    # start FULLSCREEN mode
    # On Windows, the fullscreen mode doesn't work properly, to fix the problem,
    # add with it pygame.SCALED flag
    screen = pygame.display.set_mode((1024, 768), pygame.FULLSCREEN | pygame.SCALED)
    if convert_alpha:
        background = background.convert_alpha()

    # turn off the mouse pointer
    pygame.mouse.set_visible(False)

    running = True
    cursize = [background.get_width(), background.get_height()]

    clock = pygame.Clock()

    # main loop
    while running:
        clock.tick(60)

        image = pygame.transform.smoothscale(background, cursize)
        imgpos = image.get_rect(centerx=512, centery=384)
        screen.fill((255, 255, 255))
        screen.blit(image, imgpos)

        pygame.display.flip()

        for event in pygame.event.get():
            if event.type == pygame.QUIT or (
                event.type == pygame.KEYDOWN and event.key == pygame.K_ESCAPE
            ):
                running = False

        pressed_keys = pygame.key.get_pressed()

        if pressed_keys[pygame.K_UP]:
            cursize[1] -= 2
            if cursize[1] < 1:
                cursize[1] = 1
        if pressed_keys[pygame.K_DOWN]:
            cursize[1] += 2
        if pressed_keys[pygame.K_LEFT]:
            cursize[0] -= 2
            if cursize[0] < 1:
                cursize[0] = 1
        if pressed_keys[pygame.K_RIGHT]:
            cursize[0] += 2
    pygame.quit()


def speed_test(image):
    print(f"\nImage Scaling Speed Test - Image Size {str(image.get_size())}\n")

    imgsize = [image.get_width(), image.get_height()]
    duration = 0.0
    for i in range(128):
        shrinkx = (imgsize[0] * i) // 128
        shrinky = (imgsize[1] * i) // 128
        start = time.time()
        tempimg = pygame.transform.smoothscale(image, (shrinkx, shrinky))
        duration += time.time() - start
        del tempimg

    print(f"Average transform.smoothscale shrink time: {duration / 128 * 1000:.4f} ms.")

    duration = 0.0
    for i in range(128):
        expandx = (imgsize[0] * (i + 129)) // 128
        expandy = (imgsize[1] * (i + 129)) // 128
        start = time.time()
        tempimg = pygame.transform.smoothscale(image, (expandx, expandy))
        duration += time.time() - start
        del tempimg

    print(f"Average transform.smoothscale expand time: {duration / 128 * 1000:.4f} ms.")

    duration = 0.0
    for i in range(128):
        shrinkx = (imgsize[0] * i) // 128
        shrinky = (imgsize[1] * i) // 128
        start = time.time()
        tempimg = pygame.transform.scale(image, (shrinkx, shrinky))
        duration += time.time() - start
        del tempimg

    print(f"Average transform.scale shrink time: {duration / 128 * 1000:.4f} ms.")

    duration = 0.0
    for i in range(128):
        expandx = (imgsize[0] * (i + 129)) // 128
        expandy = (imgsize[1] * (i + 129)) // 128
        start = time.time()
        tempimg = pygame.transform.scale(image, (expandx, expandy))
        duration += time.time() - start
        del tempimg

    print(f"Average transform.scale expand time: {duration / 128 * 1000:.4f} ms.")


if __name__ == "__main__":
    # check input parameters
    if len(sys.argv) < 2:
        print(f"\nUsage: {sys.argv[0]} imagefile [-t] [-convert_alpha]")
        print("    imagefile       image filename (required)")
        print("    -t              run speed test")
        print("    -convert_alpha  use convert_alpha() on the image's " "surface\n")
    else:
        main(
            sys.argv[1],
            convert_alpha="-convert_alpha" in sys.argv,
            run_speed_test="-t" in sys.argv,
        )



from typing import final
from pygame.joystick import JoystickType

def init() -> None: ...
def get_init() -> bool: ...
def quit() -> None: ...
def is_controller(device_index: int) -> bool: ...
def get_count() -> int: ...
@final
class Controller:
    def __new__(cls, device_index: int) -> Controller: ...
    def __init__(self, device_index: int) -> None: ...
    @classmethod
    def from_joystick(cls, joystick: JoystickType) -> Controller: ...
    def get_init(self) -> bool: ...
    def init(self) -> None: ...
    def quit(self) -> None: ...
    @property
    def id(self) -> int: ...
    @property
    def name(self) -> str: ...
    def attached(self) -> bool: ...
    def as_joystick(self) -> JoystickType: ...
    def get_axis(self, axis: int) -> int: ...
    def get_button(self, button: int) -> bool: ...
    def get_mapping(self) -> dict[str, str]: ...
    def set_mapping(self, mapping: dict[str, str]) -> int: ...
    def rumble(
        self, low_frequency: float, high_frequency: float, duration: int
    ) -> bool: ...
    def stop_rumble(self) -> None: ...



"""
This script is aimed at making development more convenient by having all useful
development commands under one place.

For help on how to use this, do `python dev.py -h` to get a general overview
and `python dev.py [subcommand] -h` to get subcommand specific help.
"""

import argparse
import os
import re
import subprocess
import sys
from pathlib import Path
from typing import Any, Union
from enum import Enum

MOD_NAME = "pygame-ce"
DIST_DIR = "dist"

VENV_NAME = "dev_venv"

source_tree = Path(__file__).parent
venv_path = source_tree / VENV_NAME
pyproject_path = source_tree / "pyproject.toml"

SDL3_ARGS = [
    "-Csetup-args=-Dsdl_api=3",
    "-Csetup-args=-Dmixer=disabled",
    "-Csetup-args=-Dfont=disabled",
]
COVERAGE_ARGS = ["-Csetup-args=-Dcoverage=true"]

# We assume this script works with any pip version above this.
PIP_MIN_VERSION = "23.1"


class Colors(Enum):
    RESET = "\033[0m"
    RED = "\033[31m"
    GREEN = "\033[32m"
    YELLOW = "\033[33m"
    BLUE = "\033[34m"
    MAGENTA = "\033[35m"
    CYAN = "\033[36m"
    WHITE = "\033[37m"


# logic based on https://docs.python.org/3.13/using/cmdline.html#controlling-color
def has_color():
    # highest precedence
    python_colors = os.environ.get("PYTHON_COLORS", "").strip()
    if python_colors == "1":
        return True

    if python_colors == "0":
        return False

    # second highest precedence
    if "NO_COLOR" in os.environ:
        return False

    # third highest precedence
    if "FORCE_COLOR" in os.environ:
        return True

    # lowest precedence
    return os.environ.get("TERM", "").strip().lower() != "dumb"


def pprint(arg: str, col: Colors = Colors.YELLOW):
    do_col = has_color()
    start = Colors.BLUE.value if do_col else ""
    mid = col.value if do_col else ""
    end = Colors.RESET.value if do_col else ""
    print(f"{start}[dev.py] {mid}{arg}{end}", flush=True)


def cmd_run(
    cmd: list[Union[str, Path]],
    capture_output: bool = False,
    error_on_output: bool = False,
) -> str:
    if error_on_output:
        capture_output = True

    norm_cmd = [str(i) for i in cmd]
    pprint(f"> {' '.join(norm_cmd)}", Colors.CYAN)
    try:
        ret = subprocess.run(
            norm_cmd,
            stdout=subprocess.PIPE if capture_output else sys.stdout,
            stderr=subprocess.STDOUT,
            text=capture_output,
            cwd=source_tree,
        )
    except FileNotFoundError:
        pprint(f"{norm_cmd[0]}: command not found", Colors.RED)
        sys.exit(1)

    if ret.stdout:
        print(ret.stdout, end="", flush=True)

    if (error_on_output and ret.stdout) and not ret.returncode:
        # Convert success code to failure code if we have stdout and need to
        # error
        ret.returncode = 1

    ret.check_returncode()
    return ret.stdout


def pip_install(py: Path, args: list[str]):
    return cmd_run([py, "-m", "pip", "install", "-v", *args])


def get_pyproject_list_param(section: str, key: str) -> list[str]:
    with open(pyproject_path, "r", encoding="utf-8") as f:
        content = f.read()
        if sys.version_info >= (3, 11):
            import tomllib

            cur = tomllib.loads(content)
            for i in section.split("."):
                cur = cur[i]

            return cur[key]

        # hacky solution, because we don't have tomllib in stdlib on older
        # python versions
        import ast
        import re

        # this regex only works to extract a list, nothing else
        pattern = rf"\[{section}\].*\n\s*{key}\s*=\s*(\[.*?\])"
        match = re.search(pattern, content, re.DOTALL)
        if not match:
            return []

        return ast.literal_eval(match.group(1).strip())


def get_build_deps():
    return set(get_pyproject_list_param("build-system", "requires"))


def get_cibw_setup_args():
    return [
        f"-Csetup-args={i}"
        for i in get_pyproject_list_param(
            "tool.cibuildwheel.config-settings", "setup-args"
        )
    ]


def show_diff_and_suggest_fix(parent: str):
    try:
        cmd_run(["git", "status", "--porcelain"], error_on_output=True)
    except subprocess.CalledProcessError:
        try:
            cmd_run(["git", "diff"])
        finally:
            pprint(f"Running '{parent}' caused changes")
            pprint(f"You need to run `python3 dev.py {parent}` and commit the changes")
            pprint(
                "Alternatively, you may run `python3 dev.py all` to catch more issues"
            )
        raise


def check_version_atleast(version: str, min_version: str):
    try:
        version_tup = tuple(int(i.strip()) for i in version.split("."))
        min_version_tup = tuple(int(i.strip()) for i in min_version.split("."))
    except (AttributeError, TypeError, ValueError):
        return False

    return version_tup >= min_version_tup


def check_module_in_constraint(mod: str, constraint: str):
    constraint_mod = re.match(r"[a-z0-9._-]*", constraint.lower().strip())
    if not constraint_mod:
        return False

    return mod.lower().strip() == constraint_mod[0]


class Dev:
    def __init__(self) -> None:
        self.py: Path = Path(sys.executable)
        self.args: dict[str, Any] = {}

        self.deps: dict[str, set[str]] = {
            "build": get_build_deps(),
            "docs": get_build_deps(),
            "test": {"numpy"},
            "lint": {"pylint==3.3.1", "numpy"},
            "stubs": {"mypy==1.13.0", "numpy"},
            "format": {"pre-commit==4.0.1"},
        }
        self.deps["all"] = set()
        for k in self.deps.values():
            self.deps["all"] |= k

    def cmd_build(self):
        wheel_dir = self.args.get("wheel", DIST_DIR)
        debug = self.args.get("debug", False)
        lax = self.args.get("lax", False)
        sdl3 = self.args.get("sdl3", False)
        coverage = self.args.get("coverage", False)
        if wheel_dir and coverage:
            pprint("Cannot pass --wheel and --coverage together", Colors.RED)
            sys.exit(1)

        build_suffix = ""
        if debug:
            build_suffix += "-dbg"
        if lax:
            build_suffix += "-lax"
        if sdl3:
            build_suffix += "-sdl3"
        if coverage:
            build_suffix += "-cov"
        install_args = [
            "--no-build-isolation",
            f"-Cbuild-dir=.mesonpy-build{build_suffix}",
        ]

        if not wheel_dir:
            # editable install
            install_args.append("--editable")

        install_args.append(".")

        if debug:
            install_args.append("-Csetup-args=-Dbuildtype=debug")

        if not lax:
            # use the same flags as CI
            install_args.extend(get_cibw_setup_args())

        if sdl3:
            install_args.extend(SDL3_ARGS)

        if coverage:
            install_args.extend(COVERAGE_ARGS)

        info_str = f"with {debug=}, {lax=}, {sdl3=}, and {coverage=}"
        if wheel_dir:
            pprint(f"Building wheel at '{wheel_dir}' ({info_str})")
            cmd_run(
                [self.py, "-m", "pip", "wheel", "-v", "-w", wheel_dir, *install_args]
            )
            pprint("Installing wheel")
            pip_install(
                self.py, ["--no-index", "--force", "--find-links", wheel_dir, MOD_NAME]
            )
        else:
            pprint(f"Installing in editable mode ({info_str})")
            pip_install(self.py, install_args)

    def cmd_docs(self):
        full = self.args.get("full", False)

        pprint(f"Generating docs (with {full=})")
        extra_args = ["full_generation"] if full else []
        cmd_run([self.py, "buildconfig/make_docs.py", *extra_args])

        if "CI" in os.environ:
            show_diff_and_suggest_fix("docs")

    def cmd_lint(self):
        pprint("Linting code (with pylint)")
        cmd_run([self.py, "-m", "pylint", "src_py", "docs"])

    def cmd_stubs(self):
        pprint("Generating and testing type stubs (with mypy)")
        cmd_run([self.py, "buildconfig/stubs/gen_stubs.py"])
        if "CI" in os.environ:
            show_diff_and_suggest_fix("stubs")

        cmd_run([self.py, "buildconfig/stubs/stubcheck.py"])

    def cmd_format(self):
        pre_commit = self.py.parent / "pre-commit"

        pprint("Formatting code (with pre-commit)")
        try:
            cmd_run(
                [
                    pre_commit if pre_commit.exists() else "pre-commit",
                    "run",
                    "--all-files",
                ]
            )
        except subprocess.CalledProcessError:
            # pre_commit may set error code when it modifies a file, ignore it
            pass

        if "CI" in os.environ:
            show_diff_and_suggest_fix("format")

    def cmd_test(self):
        mod = self.args.get("mod", [])

        if mod:
            pprint(f"Running tests (with module(s): {' '.join(mod)})")
            for i in mod:
                cmd_run([self.py, "-m", f"pygame.tests.{i}_test"])
        else:
            pprint("Running tests (with all modules)")
            cmd_run([self.py, "-m", "pygame.tests"])

    def cmd_all(self):
        self.cmd_format()
        self.cmd_docs()
        self.cmd_build()
        self.cmd_stubs()
        self.cmd_lint()
        self.cmd_test()

    def parse_args(self):
        parser = argparse.ArgumentParser(
            description=(
                "Build commands for the project. "
                "For more info on any subcommand you can run -h/--help on it like: "
                "dev.py build -h"
            )
        )
        subparsers = parser.add_subparsers(dest="command", required=True)

        parser.add_argument(
            "--venv",
            action="store_true",
            help="Make and use a venv (recommended, but not default)",
        )
        parser.add_argument(
            "--ignore-dep", action="append", help="Dependency to ignore in pip install"
        )

        # Build command
        build_parser = subparsers.add_parser("build", help="Build the project")
        build_parser.add_argument(
            "--wheel",
            nargs="?",
            const=DIST_DIR,  # Used if argument is provided without a value
            default="",  # Used if argument is not provided at all
            help=(
                "Generate a wheel and do a regular install from it. By default, this "
                "value is empty in which case the script does an 'editable' install. "
                "A value can passed optionally, to indicate the directory to place the "
                f"wheel (if not passed, '{DIST_DIR}' is used)"
            ),
        )
        build_parser.add_argument(
            "--debug",
            action="store_true",
            help="Install in debug mode (optimizations disabled and debug symbols enabled)",
        )
        build_parser.add_argument(
            "--lax",
            action="store_true",
            help="Be lax about build warnings, allow the build to succeed with them",
        )
        build_parser.add_argument(
            "--sdl3",
            action="store_true",
            help="Build against SDL3 instead of the default SDL2",
        )
        build_parser.add_argument(
            "--coverage",
            action="store_true",
            help=(
                "Do a coverage build. To generate a test coverage report, you need "
                "to compile pygame with this flag and run tests. This flag is only "
                "supported if the underlying compiler supports the --coverage argument"
            ),
        )

        # Docs command
        docs_parser = subparsers.add_parser("docs", help="Generate docs")
        docs_parser.add_argument(
            "--full",
            action="store_true",
            help="Force a full regeneration of docs, ignoring previous build cache",
        )

        # Test command
        test_parser = subparsers.add_parser("test", help="Run tests")
        test_parser.add_argument(
            "mod",
            nargs="*",
            help=(
                "Name(s) of sub-module(s) to test. If no args are given all are tested"
            ),
        )

        # Lint command
        subparsers.add_parser("lint", help="Lint code")

        # Stubs command
        subparsers.add_parser("stubs", help="Generate and test type stubs")

        # Format command
        subparsers.add_parser("format", help="Format code")

        # All command
        subparsers.add_parser(
            "all",
            help=(
                "Run all the subcommands. This is handy for checking that your work is "
                "ready to be submitted"
            ),
        )

        args = parser.parse_args()
        self.args = vars(args)

    def prep_env(self):
        if self.args["venv"]:
            if venv_path.is_dir():
                pprint(f"Using existing virtual environment '{venv_path}'")
            else:
                cmd_run([sys.executable, "-m", "venv", VENV_NAME])
                pprint(f"Virtual environment '{venv_path}' created")

            bin = venv_path / "Scripts" if os.name == "nt" else venv_path / "bin"
            self.py = bin / "python"
        else:
            pprint(f"Using python '{self.py}'")

        # set PATH to give high priority to executables in the python bin folder
        # this is where the binaries for meson/ninja/cython/sphinx/etc are installed
        os.environ["PATH"] = f"{self.py.parent}{os.pathsep}{os.environ.get('PATH', '')}"

        pprint("Checking pip version")
        pip_v = cmd_run([self.py, "-m", "pip", "-V"], capture_output=True)
        try:
            pip_version = pip_v.split()[1]
        except (AttributeError, IndexError):
            pip_version = "UNKNOWN"

        pprint(f"Determined pip version: {pip_version}")
        if not check_version_atleast(pip_version, PIP_MIN_VERSION):
            pprint("pip version is too old or unknown, attempting pip upgrade")
            pip_install(self.py, ["-U", "pip"])

        deps = self.deps.get(self.args["command"], set())
        ignored_deps = self.args["ignore_dep"]
        deps_filtered = deps.copy()
        if ignored_deps:
            for constr in deps:
                for dep in ignored_deps:
                    if check_module_in_constraint(dep, constr):
                        deps_filtered.remove(constr)
                        break

        if deps_filtered:
            pprint("Installing dependencies")
            pip_install(self.py, list(deps_filtered))

    def run(self):
        self.parse_args()
        self.prep_env()
        try:
            func = getattr(self, f"cmd_{self.args['command']}")
            func()
        except subprocess.CalledProcessError as e:
            pprint(f"Process exited with error code {e.returncode}", Colors.RED)
            sys.exit(e.returncode)
        except KeyboardInterrupt:
            pprint("Got KeyboardInterrupt, exiting", Colors.RED)
            sys.exit(1)

        pprint("Process exited successfully", Colors.GREEN)


if __name__ == "__main__":
    Dev().run()



import unittest
import pygame
import os
import platform

from pygame import Window
from pygame.version import SDL

pygame.init()


IS_PYPY = "PyPy" == platform.python_implementation()


class WindowTypeTest(unittest.TestCase):
    DEFAULT_TITLE = "pygame window"

    def setUp(self):
        self.win = Window()

    # Used to check the presence and set-ability of boolean attributes without
    # testing the results of the actions, since the results are not propagated
    # by the NULL videodriver on CI for some things.
    def bool_attr_test(self, attr):
        setattr(self.win, attr, True)
        setattr(self.win, attr, False)
        setattr(self.win, attr, "a truthy string")
        setattr(self.win, attr, "")
        setattr(self.win, attr, 17)
        self.assertIsInstance(getattr(self.win, attr), bool)

    def test_grab_mouse_keyboard(self):
        self.bool_attr_test("grab_mouse")
        self.bool_attr_test("grab_keyboard")

        self.win.grab_mouse = True
        self.assertTrue(self.win.grab_mouse)
        self.win.grab_mouse = False
        self.assertFalse(self.win.grab_mouse)

        if SDL >= (2, 0, 16):
            self.win.grab_keyboard = True
            self.assertTrue(self.win.grab_keyboard)
            self.win.grab_keyboard = False
            self.assertFalse(self.win.grab_keyboard)

    def test_mouse_keyboard_grabbed(self):
        self.assertIsInstance(getattr(self.win, "mouse_grabbed"), bool)
        self.assertIsInstance(getattr(self.win, "keyboard_grabbed"), bool)
        self.assertRaises(
            AttributeError, lambda: setattr(self.win, "mouse_grabbed", False)
        )
        self.assertRaises(
            AttributeError, lambda: setattr(self.win, "keyboard_grabbed", False)
        )

    def test_title(self):
        self.assertEqual(self.win.title, self.DEFAULT_TITLE)

        title = "Hello_World你好世界こにじわ"
        self.win.title = title
        self.assertEqual(self.win.title, title)

    def test_resizable(self):
        self.bool_attr_test("resizable")

    @unittest.skipIf(
        os.environ.get("SDL_VIDEODRIVER") == pygame.NULL_VIDEODRIVER,
        "requires the SDL_VIDEODRIVER to be a non-null value",
    )
    def test_resizable_set(self):
        self.win.resizable = True
        self.assertTrue(self.win.resizable)
        self.win.resizable = False
        self.assertFalse(self.win.resizable)

    def test_borderless(self):
        self.bool_attr_test("borderless")

    @unittest.skipIf(
        os.environ.get("SDL_VIDEODRIVER") == pygame.NULL_VIDEODRIVER,
        "requires the SDL_VIDEODRIVER to be a non-null value",
    )
    def test_borderless_set(self):
        self.win.borderless = True
        self.assertTrue(self.win.borderless)
        self.win.borderless = False
        self.assertFalse(self.win.borderless)

    @unittest.skipIf(
        SDL < (2, 0, 16),
        "requires SDL 2.0.16+",
    )
    def test_always_on_top(self):
        self.bool_attr_test("always_on_top")

    @unittest.skipIf(
        os.environ.get("SDL_VIDEODRIVER") == pygame.NULL_VIDEODRIVER,
        "requires the SDL_VIDEODRIVER to be a non-null value",
    )
    @unittest.skipIf(
        SDL < (2, 0, 16),
        "requires SDL 2.0.16+",
    )
    def test_always_on_top_set(self):
        self.win.always_on_top = True
        self.assertTrue(self.win.always_on_top)
        self.win.always_on_top = False
        self.assertFalse(self.win.always_on_top)

    @unittest.skipIf(
        SDL < (2, 0, 18),
        "requires SDL 2.0.18+",
    )
    def test_mouse_rect(self):
        self.win.mouse_rect = None
        self.assertIsNone(self.win.mouse_rect)

        TEST_MOUSE_RECT = (10, 10, 123, 456)
        self.win.mouse_rect = TEST_MOUSE_RECT
        self.assertIsInstance(self.win.mouse_rect, pygame.Rect)
        self.assertTupleEqual(tuple(self.win.mouse_rect), TEST_MOUSE_RECT)

        self.assertRaises(
            TypeError, lambda: setattr(self.win, "mouse_rect", "Incorrect type")
        )

        # clean the status
        self.win.mouse_rect = None

    def test_size(self):
        self.win.size = (1280, 720)
        self.assertTupleEqual(self.win.size, (1280, 720))

        self.assertRaises(ValueError, lambda: setattr(self.win, "size", (-1, 10)))
        self.assertRaises(ValueError, lambda: setattr(self.win, "size", (1, -10)))
        self.assertRaises(ValueError, lambda: setattr(self.win, "size", (-1, -10)))
        self.assertRaises(ValueError, lambda: setattr(self.win, "size", (0, 0)))

        # test set size when init
        # On Windows, if SDL <= 2.0.22, this test fails
        # This bug have been fixed in SDL 2.24.0
        if SDL > (2, 0, 22):
            win = Window(size=(12, 34))
            self.assertTupleEqual((12, 34), win.size)
            win.destroy()

        self.assertRaises(ValueError, lambda: Window(size=(-1, 0)))

        self.win.size = (640, 480)

    def test_position(self):
        self.win.position = (12, 34)
        self.assertTupleEqual(self.win.position, (12, 34))

        self.win.position = pygame.WINDOWPOS_CENTERED

        # raise TypeError if input is some random number instead of flag
        self.assertRaises(TypeError, lambda: setattr(self.win, "position", 123))

        # test set position when init
        win = Window(position=(20, 48))
        self.assertTupleEqual((20, 48), win.position)
        win.destroy()

        self.assertRaises(TypeError, lambda: Window(position=123))

    def test_maximum_size(self):
        # Use a new window to prevent being influenced by other tests
        self.win = Window()

        self.win.maximum_size = (50, 50)
        self.assertTupleEqual(self.win.maximum_size, (50, 50))

        # size should be limited to the maximum_size
        self.assertTupleEqual(self.win.size, (50, 50))

        self.win.size = (640, 480)
        # size should not larger than maximum_size
        self.assertTupleEqual(self.win.size, (50, 50))

        if SDL.major >= 3:
            # This part should pass in SDL3
            self.win.maximum_size = (0, 0)  # remove the size limit
            self.win.size = (640, 480)
            self.assertTupleEqual(self.win.size, (640, 480))

        self.assertRaises(
            ValueError, lambda: setattr(self.win, "maximum_size", (-1, -1))
        )
        self.assertRaises(
            TypeError, lambda: setattr(self.win, "maximum_size", "placeholder_str")
        )

    def test_minimum_size(self):
        # Use a new window to prevent being influenced by other tests
        self.win = Window()

        self.win.minimum_size = (700, 700)
        self.assertTupleEqual(self.win.minimum_size, (700, 700))

        # size should be limited to the minimum_size
        self.assertTupleEqual(self.win.size, (700, 700))

        self.win.size = (640, 480)
        # size should not small than minimum_size
        self.assertTupleEqual(self.win.size, (700, 700))

        if SDL.major >= 3:
            # This part should pass in SDL3
            self.win.minimum_size = (0, 0)  # remove the size limit
            self.win.size = (640, 480)
            self.assertTupleEqual(self.win.size, (640, 480))

        self.assertRaises(
            ValueError, lambda: setattr(self.win, "minimum_size", (-1, -1))
        )
        self.assertRaises(
            TypeError, lambda: setattr(self.win, "minimum_size", "null_str")
        )

    def test_min_size_interact_with_max_size(self):
        # Use a new window to prevent being influenced by other tests
        self.win = Window()

        self.win.maximum_size = (100, 100)
        # min size should not larger than max size
        self.assertRaises(
            ValueError, lambda: setattr(self.win, "minimum_size", (200, 200))
        )

        self.win.minimum_size = (90, 90)
        # max size should not smaller than min size
        self.assertRaises(
            ValueError, lambda: setattr(self.win, "maximum_size", (50, 50))
        )

        # minimum size should be able to equal to maximum size
        # This test fails in SDL <= 2.0.12
        # have been fixed after SDL 2.0.18
        if SDL >= (2, 0, 18):
            self.win.minimum_size = (60, 60)
            self.win.maximum_size = (60, 60)
            self.assertTupleEqual(self.win.maximum_size, (60, 60))
            self.assertTupleEqual(self.win.minimum_size, (60, 60))

    def test_opacity(self):
        # Setting is not supported at all with SDL_VIDEODRIVER = pygame.NULL_VIDEODRIVER,
        # all we can test here is that the attribute exists and is a
        # float
        self.assertIsInstance(self.win.opacity, float)

    @unittest.skipIf(
        os.environ.get("SDL_VIDEODRIVER") == pygame.NULL_VIDEODRIVER,
        "requires the SDL_VIDEODRIVER to be a non-null value",
    )
    def test_opacity_set(self):
        self.win.opacity = 0.5
        self.assertEqual(self.win.opacity, 0.5)
        self.win.opacity = 0
        self.assertEqual(self.win.opacity, 0)
        self.win.opacity = 1
        self.assertEqual(self.win.opacity, 1)
        self.win.opacity = -0.1
        self.assertEqual(self.win.opacity, 0)
        self.win.opacity = 1.1
        self.assertEqual(self.win.opacity, 1)

        self.assertRaises(TypeError, lambda: setattr(self.win, "opacity", "null str"))

    def test_init_flags(self):
        # test no opengl by default
        win = Window()
        self.assertFalse(win.opengl)
        win.destroy()

        # test borderless
        win = Window(borderless=True)
        self.assertTrue(win.borderless)
        win.destroy()

        # test always_on_top
        if SDL >= (2, 0, 16):
            win = Window(always_on_top=True)
            self.assertTrue(win.always_on_top)
            win.destroy()

        # test resizable
        win = Window(resizable=True)
        self.assertTrue(win.resizable)
        win.destroy()

        # test utility
        win = Window(utility=True)
        self.assertTrue(win.utility)
        win.destroy()

        # should raise a TypeError if keyword is random
        self.assertRaises(TypeError, lambda: Window(aaa=True))
        self.assertRaises(TypeError, lambda: Window(aaa=False))

    def test_set_icon(self):
        self.assertRaises(TypeError, lambda: self.win.set_icon(1234))

        test_icon = pygame.Surface((32, 32))
        test_icon.fill((255, 0, 0))

        return_value = self.win.set_icon(test_icon)
        self.assertIsNone(return_value)

    def test_window_object_repr(self):
        win = Window("awa")
        self.assertEqual(repr(win), f"<Window(title='{win.title}', id={win.id})>")

        win.destroy()
        self.assertEqual(repr(win), "<Window(Destroyed)>")

        pygame.display.set_mode((640, 480))
        win = Window.from_display_module()
        self.assertEqual(repr(win), "<Window(From Display)>")

        pygame.display.quit()
        pygame.init()

    def test_from_display_module(self):
        pygame.display.set_mode((640, 480))

        win1 = Window.from_display_module()
        win2 = Window.from_display_module()

        self.assertIs(win1, win2)
        self.assertFalse(win1.opengl)

        pygame.display.quit()
        pygame.init()

    @unittest.skipIf(IS_PYPY, "for some reason this test is flaky on pypy")
    def test_window_surface(self):
        win = Window(size=(640, 480))
        surf = win.get_surface()

        self.assertIsInstance(surf, pygame.Surface)

        # test auto resize
        self.assertTupleEqual(win.size, surf.get_size())
        win.size = (100, 100)
        self.assertTupleEqual(win.size, surf.get_size())
        win.size = (1280, 720)
        self.assertTupleEqual(win.size, surf.get_size())

        # window surface should be invalid after the window is destroyed
        win.destroy()
        self.assertRaises(pygame.error, lambda: surf.fill((0, 0, 0)))

    @unittest.skipIf(IS_PYPY, "for some reason this test is flaky on pypy")
    def test_window_surface_with_display_module(self):
        # get_surface() should raise an error if the set_mode() is not called.
        pygame.display.set_mode((640, 480))
        win1 = Window.from_display_module()
        pygame.display.quit()
        pygame.init()
        self.assertRaises(pygame.error, lambda: win1.get_surface())

        # the surface returned by get_surface() should be
        # the surface returned by set_mode()
        surf1 = pygame.display.set_mode((640, 480))
        win2 = Window.from_display_module()
        surf2 = win2.get_surface()
        self.assertIs(surf1, surf2)

    def test_window_flip(self):
        win = Window(size=(640, 480))
        surf = win.get_surface()
        surf.fill((255, 0, 0))

        self.assertRaises(TypeError, lambda: win.flip("an argument"))
        self.assertIs(win.flip(), None)
        win.destroy()

        # creates a new window with no surface associated
        win = Window(size=(640, 480))
        self.assertRaisesRegex(
            pygame.error,
            "the Window has no surface associated with it, did you forget to call Window.get_surface()",
            win.flip,
        )
        win.destroy()

    @unittest.skipIf(
        os.environ.get("SDL_VIDEODRIVER") == pygame.NULL_VIDEODRIVER,
        "OpenGL requires a non-null SDL_VIDEODRIVER",
    )
    def test_window_opengl(self):
        win1 = Window(opengl=True)
        self.assertTrue(win1.opengl)
        win1.flip()
        win1.destroy()

        win2 = Window(opengl=False)
        self.assertFalse(win2.opengl)
        win2.get_surface()
        win2.flip()
        win2.destroy()

        pygame.display.set_mode((640, 480), pygame.OPENGL)
        win = Window.from_display_module()
        self.assertTrue(win.opengl)
        pygame.display.quit()
        pygame.init()

    @unittest.skipIf(IS_PYPY, "for some reason this test is flaky on pypy")
    def test_window_subclassable(self):
        class WindowSubclass(Window):
            def __init__(self, title="Different title", size=(640, 480), **flags):
                super().__init__(title, size, pygame.WINDOWPOS_CENTERED, **flags)
                self.attribute = 10

        window = WindowSubclass()
        self.assertTrue(issubclass(WindowSubclass, Window))
        self.assertIsInstance(window, WindowSubclass)
        self.assertEqual(window.title, "Different title")
        self.assertEqual(window.attribute, 10)
        window.destroy()

        pygame.display.set_mode((200, 200))
        window = WindowSubclass.from_display_module()
        self.assertIsInstance(window, WindowSubclass)
        self.assertEqual(window.size, (200, 200))

    @unittest.skipIf(
        SDL < (2, 0, 16),
        "requires SDL 2.0.16+",
    )
    def test_window_flash(self):
        window = pygame.Window()

        with self.assertRaises(TypeError):
            window.flash("string")
            window.flash(2.2)
            window.flash([0])

        with self.assertRaises(ValueError):
            window.flash(-1)
            window.flash(3)

        for operation in [
            pygame.FLASH_CANCEL,
            pygame.FLASH_BRIEFLY,
            pygame.FLASH_UNTIL_FOCUSED,
        ]:
            try:
                result = window.flash(operation)
                self.assertIsNone(result)
            except pygame.error:
                pass

    def test_window_focused(self):
        window = pygame.Window()
        self.assertIsInstance(window.focused, bool)

    def tearDown(self):
        self.win.destroy()


if __name__ == "__main__":
    unittest.main()



"""Set of common pygame type aliases for proper typehint annotations"""

# NOTE: `src_py/typing.py` and `buildconfig/stubs/pygame/typing.pyi` must be duplicates.
# Use the command `python buildconfig/stubs/gen_stubs.py` to copy typing.py to typing.pyi

__all__ = [
    "RectLike",
    "SequenceLike",
    "FileLike",
    "ColorLike",
    "Point",
    "IntPoint",
]

from abc import abstractmethod
from collections.abc import Callable
from os import PathLike as _PathProtocol
from typing import IO, Union, TypeVar, Protocol

from pygame.color import Color
from pygame.rect import Rect, FRect


# For functions that take a file name
_PathLike = Union[str, bytes, _PathProtocol[str], _PathProtocol[bytes]]
# Most pygame functions that take a file argument should be able to handle a FileLike type
FileLike = Union[_PathLike, IO[bytes], IO[str]]

_T_co = TypeVar("_T_co", covariant=True)


class SequenceLike(Protocol[_T_co]):
    """
    Variant of the standard `Sequence` ABC that only requires `__getitem__` and `__len__`.
    """

    @abstractmethod
    def __getitem__(self, index: int, /) -> _T_co: ...
    @abstractmethod
    def __len__(self) -> int: ...


# Modify typehints when it is possible to annotate sizes

# Pygame handles float without errors in most cases where a point is expected,
# usually rounding to int. Also, 'Union[int, float] == float'
Point = SequenceLike[float]
# This is used where ints are strictly required
IntPoint = SequenceLike[int]

ColorLike = Union[Color, SequenceLike[int], str, int]


class _HasRectAttribute(Protocol):
    # An object that has a rect attribute that is either a rect, or a function
    # that returns a rect conforms to the rect protocol
    @property
    def rect(self) -> Union["RectLike", Callable[[], "RectLike"]]: ...


RectLike = Union[
    Rect, FRect, SequenceLike[float], SequenceLike[Point], _HasRectAttribute
]


# cleanup namespace
del (
    abstractmethod,
    Color,
    Rect,
    FRect,
    IO,
    Callable,
    Union,
    TypeVar,
    Protocol,
)



#!/usr/bin/env python
"""pygame.examples.video

Experimental!

* dialog message boxes with messagebox.
* multiple windows with Window
* driver selection
* Renderer, Texture, and Image classes
* Drawing lines, rects, and such onto Renderers.
"""

import os
import pygame

from pygame._sdl2 import Window, Texture, Image, Renderer, get_drivers

data_dir = os.path.join(os.path.split(os.path.abspath(__file__))[0], "data")


def load_img(file):
    return pygame.image.load(os.path.join(data_dir, file))


def main():
    pygame.display.init()
    pygame.key.set_repeat(1000, 10)

    for driver in get_drivers():
        print(driver)

    import random

    answer = pygame.display.message_box(
        "I will open two windows! Continue?",
        "Hello!",
        message_type="info",
        buttons=("Yes", "No", "Chance"),
        return_button=0,
        escape_button=1,
    )

    if answer == 1 or (answer == 2 and random.random() < 0.5):
        import sys

        sys.exit(0)

    win = Window("asdf", resizable=True)
    renderer = Renderer(win)
    tex = Texture.from_surface(renderer, load_img("alien1.gif"))

    running = True

    x, y = 250, 50
    clock = pygame.Clock()

    backgrounds = [(255, 0, 0, 255), (0, 255, 0, 255), (0, 0, 255, 255)]
    bg_index = 0

    renderer.draw_color = backgrounds[bg_index]

    win2 = Window("2nd window", size=(256, 256), always_on_top=True)
    win2.opacity = 0.5
    win2.set_icon(load_img("bomb.gif"))
    renderer2 = Renderer(win2)
    tex2 = Texture.from_surface(renderer2, load_img("asprite.bmp"))
    renderer2.clear()
    tex2.draw()
    renderer2.present()
    del tex2

    full = 0

    tex = Image(tex)

    surf = pygame.Surface((64, 64))
    streamtex = Texture(renderer, (64, 64), streaming=True)
    tex_update_interval = 1000
    next_tex_update = pygame.time.get_ticks()

    while running:
        for event in pygame.event.get():
            if event.type == pygame.QUIT:
                running = False
            elif getattr(event, "window", None) == win2:
                if (
                    event.type == pygame.KEYDOWN
                    and event.key == pygame.K_ESCAPE
                    or event.type == pygame.WINDOWCLOSE
                ):
                    win2.destroy()
            elif event.type == pygame.KEYDOWN:
                if event.key == pygame.K_ESCAPE:
                    running = False
                elif event.key == pygame.K_LEFT:
                    x -= 5
                elif event.key == pygame.K_RIGHT:
                    x += 5
                elif event.key == pygame.K_DOWN:
                    y += 5
                elif event.key == pygame.K_UP:
                    y -= 5
                elif event.key == pygame.K_f:
                    if full == 0:
                        win.set_fullscreen(True)
                        full = 1
                    else:
                        win.set_windowed()
                        full = 0
                elif event.key == pygame.K_s:
                    readsurf = renderer.to_surface()
                    pygame.image.save(readsurf, "test.png")

                elif event.key == pygame.K_SPACE:
                    bg_index = (bg_index + 1) % len(backgrounds)
                    renderer.draw_color = backgrounds[bg_index]

        renderer.clear()

        # update texture
        curtime = pygame.time.get_ticks()
        if curtime >= next_tex_update:
            for x_ in range(streamtex.width // 4):
                for y_ in range(streamtex.height // 4):
                    newcol = (
                        random.randint(0, 255),
                        random.randint(0, 255),
                        random.randint(0, 255),
                        255,
                    )
                    area = (4 * x_, 4 * y_, 4, 4)
                    surf.fill(newcol, area)
            streamtex.update(surf)
            next_tex_update = curtime + tex_update_interval
        streamtex.draw(dstrect=pygame.Rect(64, 128, 64, 64))

        tex.draw(dstrect=(x, y))

        # TODO: should these be?
        # - line instead of draw_line
        # - point instead of draw_point
        # - rect(rect, width=1)->draw 1 pixel, instead of draw_rect
        # - rect(rect, width=0)->filled ? , instead of fill_rect
        #
        # TODO: should these work with pygame.draw.line(renderer, ...) functions?
        renderer.draw_color = (255, 255, 255, 255)
        renderer.draw_line((0, 0), (64, 64))
        renderer.draw_line((64, 64), (128, 0))
        renderer.draw_point((72, 32))
        renderer.draw_rect(pygame.Rect(0, 64, 64, 64))
        renderer.fill_rect(pygame.Rect(0, 128, 64, 64))
        renderer.draw_color = backgrounds[bg_index]

        renderer.present()

        clock.tick(60)
        win.title = str(f"FPS: {clock.get_fps()}")

    pygame.quit()


if __name__ == "__main__":
    main()



import warnings

warnings.warn(
    "The ._sprite submodule is deprecated since 2.4.0. Please use pygame.sprite instead.",
    category=DeprecationWarning,
    stacklevel=2,
)

from pygame.sprite import (
    Sprite,
    DirtySprite,
    Group,
    GroupSingle,
    AbstractGroup,
    RenderClear,
    RenderPlain,
    RenderUpdates,
    LayeredUpdates,
    LayeredDirty,
    spritecollide,
    spritecollideany,
    collide_mask,
    collide_rect,
    collide_circle,
    collide_rect_ratio,
    collide_circle_ratio,
    groupcollide,
)



#!/usr/bin/env python
"""pygame.examples.audiocapture

A pygame 2 experiment.

* record sound from a microphone
* play back the recorded sound
"""

import pygame
import time

from pygame._sdl2 import (
    get_audio_device_names,
    AudioDevice,
    AUDIO_F32,
    AUDIO_ALLOW_FORMAT_CHANGE,
)
from pygame._sdl2.mixer import set_post_mix

sounds = []
sound_chunks = []


def callback(audiodevice, audiomemoryview):
    """This is called in the sound thread.

    Note, that the frequency and such you request may not be what you get.
    """
    # print(type(audiomemoryview), len(audiomemoryview))
    # print(audiodevice)
    sound_chunks.append(bytes(audiomemoryview))


def postmix_callback(postmix, audiomemoryview):
    """This is called in the sound thread.

    At the end of mixing we get this data.
    """
    print(type(audiomemoryview), len(audiomemoryview))
    print(postmix)


def main():
    pygame.mixer.pre_init(44100, 32, 2, 512)
    pygame.init()

    # init_subsystem(INIT_AUDIO)
    names = get_audio_device_names(True)
    print(names)

    set_post_mix(postmix_callback)

    audio = AudioDevice(
        devicename=names[0],
        iscapture=True,
        frequency=44100,
        audioformat=AUDIO_F32,
        numchannels=2,
        chunksize=512,
        allowed_changes=AUDIO_ALLOW_FORMAT_CHANGE,
        callback=callback,
    )
    # start recording.
    audio.pause(0)

    print(audio)

    print(f"recording with '{names[0]}'")
    time.sleep(5)

    print("Turning data into a pygame.mixer.Sound")
    sound = pygame.mixer.Sound(buffer=b"".join(sound_chunks))

    print("playing back recorded sound")
    sound.play()
    time.sleep(5)
    pygame.quit()


if __name__ == "__main__":
    main()



from pygame.typing import RectLike

class ScancodeWrapper(tuple[bool, ...]): ...

def get_focused() -> bool: ...
def get_pressed() -> ScancodeWrapper: ...
def get_just_pressed() -> ScancodeWrapper: ...
def get_just_released() -> ScancodeWrapper: ...
def get_mods() -> int: ...
def set_mods(mods: int, /) -> None: ...
def set_repeat(delay: int = 0, interval: int = 0, /) -> None: ...
def get_repeat() -> tuple[int, int]: ...
def name(key: int, use_compat: bool = True) -> str: ...
def key_code(name: str) -> int: ...
def start_text_input() -> None: ...
def stop_text_input() -> None: ...
def set_text_input_rect(rect: RectLike, /) -> None: ...



from typing_extensions import deprecated # added in 3.13

import numpy

from pygame.mixer import Sound

def array(sound: Sound) -> numpy.ndarray: ...
def samples(sound: Sound) -> numpy.ndarray: ...
def make_sound(array: numpy.ndarray) -> Sound: ...
@deprecated("Only numpy is supported")
def use_arraytype(arraytype: str) -> Sound: ...
@deprecated("Only numpy is supported")
def get_arraytype() -> str: ...
@deprecated("Only numpy is supported")
def get_arraytypes() -> tuple[str]: ...



# python -m pygame.docs.serve

import sys
import webbrowser
from http.server import ThreadingHTTPServer, SimpleHTTPRequestHandler

# use relative import here and not absolute, so that `python -m docs` works at
# development time
from . import PKG_DIR, has_local_docs


class DocsHandler(SimpleHTTPRequestHandler):
    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs, directory=str(PKG_DIR))


def serve(address: str, port: int):
    with ThreadingHTTPServer((address, port), DocsHandler) as httpd:
        try:
            httpd.serve_forever()
        except KeyboardInterrupt:
            print("\nKeyboard interrupt received, exiting.")
            sys.exit(0)


TARGET = "localhost"


def main():
    print("Running a simple server to serve documentation.")
    print(
        "If you want to just open documentation files in the browser, "
        "run 'python -m pygame.docs.static'."
    )
    if not has_local_docs():
        print("ERROR: no local documentation found, cannot serve anything, exiting...")
        sys.exit(1)
    print("WARNING: this is not for production use!")
    import argparse

    parser = argparse.ArgumentParser()
    parser.add_argument(
        "port",
        action="store",
        default=8000,
        type=int,
        nargs="?",
        help="specify alternate port (default: 8000)",
    )
    parser.add_argument(
        "--no-browser",
        "-n",
        action="store_false",
        default=True,
        dest="open_browser",
        help="whether to open a browser tab",
    )

    parsed_args = parser.parse_args()

    print(f"Serving on: http://{TARGET}:{parsed_args.port}")

    if parsed_args.open_browser:
        webbrowser.open(f"http://{TARGET}:{parsed_args.port}")

    serve(TARGET, parsed_args.port)


if __name__ == "__main__":
    main()



# cython: language_level=3str
#

from .sdl2 cimport *

cdef extern from "pgcompat_rect.h" nogil:
    ctypedef struct SDL_FRect:
        float x, y
        float w, h


cdef extern from "SDL.h" nogil:
    ctypedef struct SDL_Window
    ctypedef struct SDL_Texture
    ctypedef struct SDL_Renderer
    ctypedef struct SDL_Rect:
        int x, y
        int w, h

    ctypedef enum SDL_PixelFormatEnum:
        SDL_PIXELFORMAT_UNKNOWN

    int SDL_BITSPERPIXEL(Uint32 format)

    ctypedef struct SDL_PixelFormat:
        Uint32 format

    ctypedef struct SDL_Surface:
        Uint32 flags
        SDL_PixelFormat *format
        int w,h
        int pitch
        void* pixels
        void *userdata
        int locked
        void *lock_data
        SDL_Rect clip_rect

    ctypedef struct SDL_Point:
        int x, y
    ctypedef struct SDL_Color:
        Uint8 r, g, b, a

    SDL_bool SDL_SetHint(const char *name, const char *value)
    char* SDL_HINT_RENDER_SCALE_QUALITY b'SDL_HINT_RENDER_SCALE_QUALITY'

    cdef extern from *:
        """
        #if SDL_VERSION_ATLEAST(2, 0, 18)
            typedef SDL_FPoint _pgsdlFPoint;
            typedef SDL_Vertex _pgsdlVertex;
        #else
            typedef struct {
                float x;
                float y;
            } _pgsdlFPoint;

            typedef struct {
                _pgsdlFPoint position;
                SDL_Color color;
                _pgsdlFPoint tex_coord;
            } _pgsdlVertex;

            int SDL_RenderGeometry(SDL_Renderer *renderer, SDL_Texture *texture,
                  const _pgsdlVertex *vertices, int num_vertices,
                  const int *indices, int num_indices)
            {
                SDL_SetError("Needs SDL 2.0.18");
                return -1;
            }
        #endif

        #if SDL_VERSION_ATLEAST(2,0,12)
            typedef SDL_ScaleMode _pgsdlScaleMode;
        #else
            typedef enum {
                SDL_ScaleModeNearest,
                SDL_ScaleModeLinear,
                SDL_ScaleModeBest,
            } _pgsdlScaleMode;

            int SDL_SetTextureScaleMode(SDL_Texture * texture, _pgsdlScaleMode scaleMode){
                return 0;
            }
        #endif
        """
    ctypedef struct SDL_FPoint "_pgsdlFPoint":
        float x, y
    ctypedef struct SDL_Vertex "_pgsdlVertex":
        SDL_FPoint position
        SDL_Color color
        SDL_FPoint tex_coord

    ctypedef enum SDL_ScaleMode "_pgsdlScaleMode":
        SDL_ScaleModeNearest,
        SDL_ScaleModeLinear,
        SDL_ScaleModeBest

    int SDL_SetTextureScaleMode(SDL_Texture * texture, SDL_ScaleMode scaleMode)

    ctypedef enum SDL_RendererFlip:
        SDL_FLIP_NONE,
        SDL_FLIP_HORIZONTAL,
        SDL_FLIP_VERTICAL
    ctypedef enum SDL_BlendMode:
        SDL_BLENDMODE_NONE = 0x00000000,
        SDL_BLENDMODE_BLEND = 0x00000001,
        SDL_BLENDMODE_ADD = 0x00000002,
        SDL_BLENDMODE_MOD = 0x00000004,
        SDL_BLENDMODE_MUL = 0x00000008,
        SDL_BLENDMODE_INVALID = 0x7FFFFFFF

    # https://wiki.libsdl.org/SDL_MessageBoxData
    # https://wiki.libsdl.org/SDL_ShowMessageBox
    cdef Uint32 _SDL_MESSAGEBOX_ERROR "SDL_MESSAGEBOX_ERROR"
    cdef Uint32 _SDL_MESSAGEBOX_WARNING "SDL_MESSAGEBOX_WARNING"
    cdef Uint32 _SDL_MESSAGEBOX_INFORMATION "SDL_MESSAGEBOX_INFORMATION"

    cdef Uint32 _SDL_MESSAGEBOX_BUTTON_RETURNKEY_DEFAULT "SDL_MESSAGEBOX_BUTTON_RETURNKEY_DEFAULT"
    cdef Uint32 _SDL_MESSAGEBOX_BUTTON_ESCAPEKEY_DEFAULT "SDL_MESSAGEBOX_BUTTON_ESCAPEKEY_DEFAULT"

    ctypedef struct SDL_MessageBoxData:
        Uint32 flags
        SDL_Window* window
        const char* title
        const char* message
        int numbuttons
        const SDL_MessageBoxButtonData* buttons
        const SDL_MessageBoxColorScheme* colorScheme
    ctypedef struct SDL_MessageBoxButtonData:
        Uint32 flags
        int buttonid
        const char *text
    ctypedef struct SDL_MessageBoxColorScheme
    int SDL_ShowMessageBox(const SDL_MessageBoxData* messageboxdata,
                           int*                      buttonid)

    # RENDERER
    cdef Uint32 _SDL_RENDERER_SOFTWARE "SDL_RENDERER_SOFTWARE"
    cdef Uint32 _SDL_RENDERER_ACCELERATED "SDL_RENDERER_ACCELERATED"
    cdef Uint32 _SDL_RENDERER_PRESENTVSYNC "SDL_RENDERER_PRESENTVSYNC"
    cdef Uint32 _SDL_RENDERER_TARGETTEXTURE "SDL_RENDERER_TARGETTEXTURE"

    # https://wiki.libsdl.org/SDL_SetRenderDrawColor
    # https://wiki.libsdl.org/SDL_SetRenderDrawBlendMode
    # https://wiki.libsdl.org/SDL_GetRenderDrawBlendMode
    # https://wiki.libsdl.org/SDL_CreateRenderer
    # https://wiki.libsdl.org/SDL_DestroyRenderer
    # https://wiki.libsdl.org/SDL_RenderClear
    # https://wiki.libsdl.org/SDL_RenderCopy
    # https://wiki.libsdl.org/SDL_RenderCopyEx
    # https://wiki.libsdl.org/SDL_RenderCopyExF
    # https://wiki.libsdl.org/SDL_RenderPresent
    int SDL_GetRenderDrawColor(SDL_Renderer* renderer,
                               Uint8*        r,
                               Uint8*        g,
                               Uint8*        b,
                               Uint8*        a)
    int SDL_SetRenderDrawColor(SDL_Renderer* renderer,
                               Uint8         r,
                               Uint8         g,
                               Uint8         b,
                               Uint8         a)
    int SDL_GetRenderDrawBlendMode(SDL_Renderer*   renderer,
                                   SDL_BlendMode* blendMode)
    int SDL_SetRenderDrawBlendMode(SDL_Renderer*  renderer,
                                   SDL_BlendMode blendMode)
    SDL_Renderer* SDL_CreateRenderer(SDL_Window* window,
                                     int         index,
                                     Uint32      flags)
    void SDL_DestroyRenderer(SDL_Renderer* renderer)
    int SDL_RenderClear(SDL_Renderer* renderer)
    int SDL_RenderCopy(SDL_Renderer*   renderer,
                       SDL_Texture*    texture,
                       const SDL_Rect* srcrect,
                       const SDL_Rect* dstrect)
    int SDL_RenderCopyEx(SDL_Renderer*          renderer,
                         SDL_Texture*           texture,
                         const SDL_Rect*        srcrect,
                         const SDL_Rect*        dstrect,
                         const double           angle,
                         const SDL_Point*       center,
                         const SDL_RendererFlip flip)
    int SDL_RenderCopyExF(SDL_Renderer*          renderer,
                          SDL_Texture*           texture,
                          const SDL_Rect*        srcrect,
                          const SDL_FRect*       dstrect,
                          const double           angle,
                          const SDL_FPoint*      center,
                          const SDL_RendererFlip flip)
    void SDL_RenderPresent(SDL_Renderer* renderer)
    # https://wiki.libsdl.org/SDL_RenderGetViewport
    # https://wiki.libsdl.org/SDL_RenderSetViewport
    void SDL_RenderGetViewport(SDL_Renderer* renderer,
                               SDL_Rect*     rect)
    int SDL_RenderSetViewport(SDL_Renderer*   renderer,
                              const SDL_Rect* rect)
    # https://wiki.libsdl.org/SDL_RenderReadPixels
    int SDL_RenderReadPixels(SDL_Renderer*   renderer,
                             const SDL_Rect* rect,
                             Uint32          format,
                             void*           pixels,
                             int             pitch)
    # https://wiki.libsdl.org/SDL_SetRenderTarget
    int SDL_SetRenderTarget(SDL_Renderer* renderer,
                            SDL_Texture*  texture)

    # https://wiki.libsdl.org/SDL_RendererInfo
    ctypedef struct SDL_RendererInfo:
        const char *name
        Uint32 flags
        Uint32 num_texture_formats
        Uint32[16] texture_formats
        int max_texture_width
        int max_texture_height
    # https://wiki.libsdl.org/SDL_GetNumRenderDrivers
    int SDL_GetNumRenderDrivers()
    # https://wiki.libsdl.org/SDL_GetRenderDriverInfo
    int SDL_GetRenderDriverInfo(int               index,
                                SDL_RendererInfo* info)

    # https://wiki.libsdl.org/SDL_ComposeCustomBlendMode
    # https://wiki.libsdl.org/SDL_BlendFactor
    # https://wiki.libsdl.org/SDL_BlendOperation
    SDL_BlendMode SDL_ComposeCustomBlendMode(SDL_BlendFactor    srcColorFactor,
                                             SDL_BlendFactor    dstColorFactor,
                                             SDL_BlendOperation colorOperation,
                                             SDL_BlendFactor    srcAlphaFactor,
                                             SDL_BlendFactor    dstAlphaFactor,
                                             SDL_BlendOperation alphaOperation)

    ctypedef enum SDL_BlendOperation:
        SDL_BLENDOPERATION_ADD = 0x00000001,
        SDL_BLENDOPERATION_SUBTRACT = 0x00000002,
        SDL_BLENDOPERATION_REV_SUBTRACT = 0x00000003,
        SDL_BLENDOPERATION_MINIMUM = 0x00000004,
        SDL_BLENDOPERATION_MAXIMUM = 0x00000005

    ctypedef enum SDL_BlendFactor:
        SDL_BLENDFACTOR_ZERO = 0x00000001,
        SDL_BLENDFACTOR_ONE = 0x00000002,
        SDL_BLENDFACTOR_SRC_COLOR = 0x00000003,
        SDL_BLENDFACTOR_ONE_MINUS_SRC_COLOR = 0x00000004,
        SDL_BLENDFACTOR_SRC_ALPHA = 0x00000005,
        SDL_BLENDFACTOR_ONE_MINUS_SRC_ALPHA = 0x00000006,
        SDL_BLENDFACTOR_DST_COLOR = 0x00000007,
        SDL_BLENDFACTOR_ONE_MINUS_DST_COLOR = 0x00000008,
        SDL_BLENDFACTOR_DST_ALPHA = 0x00000009,
        SDL_BLENDFACTOR_ONE_MINUS_DST_ALPHA = 0x0000000A

    # WINDOW
    # https://wiki.libsdl.org/SDL_GetWindowData
    # https://wiki.libsdl.org/SDL_GetGrabbedWindow
    void* SDL_GetWindowData(SDL_Window* window,
                            const char* name)
    SDL_Window* SDL_GetGrabbedWindow()

    cdef int _SDL_WINDOWPOS_UNDEFINED "SDL_WINDOWPOS_UNDEFINED"
    cdef int _SDL_WINDOWPOS_CENTERED "SDL_WINDOWPOS_CENTERED"

    # TEXTURE
    # https://wiki.libsdl.org/SDL_CreateTexture
    # https://wiki.libsdl.org/SDL_CreateTextureFromSurface
    # https://wiki.libsdl.org/SDL_DestroyTexture
    # https://wiki.libsdl.org/SDL_GetTextureAlphaMod
    # https://wiki.libsdl.org/SDL_SetTextureAlphaMod
    # https://wiki.libsdl.org/SDL_GetTextureBlendMode
    # https://wiki.libsdl.org/SDL_SetTextureBlendMode
    # https://wiki.libsdl.org/SDL_GetTextureColorMod
    # https://wiki.libsdl.org/SDL_SetTextureColorMod
    SDL_Texture* SDL_CreateTexture(SDL_Renderer* renderer,
                                   Uint32        format,
                                   int           access,
                                   int           w,
                                   int           h)
    SDL_Texture* SDL_CreateTextureFromSurface(SDL_Renderer* renderer,
                                              SDL_Surface*  surface)
    void SDL_DestroyTexture(SDL_Texture* texture)
    # https://wiki.libsdl.org/SDL_TextureAccess
    cdef Uint32 _SDL_TEXTUREACCESS_STATIC "SDL_TEXTUREACCESS_STATIC"
    cdef Uint32 _SDL_TEXTUREACCESS_STREAMING "SDL_TEXTUREACCESS_STREAMING"
    cdef Uint32 _SDL_TEXTUREACCESS_TARGET "SDL_TEXTUREACCESS_TARGET"

    Uint32 SDL_MasksToPixelFormatEnum(int    bpp,
                                      Uint32 Rmask,
                                      Uint32 Gmask,
                                      Uint32 Bmask,
                                      Uint32 Amask)


    int SDL_GetTextureAlphaMod(SDL_Texture* texture,
                               Uint8*       alpha)
    int SDL_SetTextureAlphaMod(SDL_Texture* texture,
                               Uint8        alpha)

    int SDL_GetTextureBlendMode(SDL_Texture*   texture,
                                SDL_BlendMode* blendMode)
    int SDL_SetTextureBlendMode(SDL_Texture*  texture,
                                SDL_BlendMode blendMode)
    int SDL_GetTextureColorMod(SDL_Texture* texture,
                               Uint8*       r,
                               Uint8*       g,
                               Uint8*       b)
    int SDL_SetTextureColorMod(SDL_Texture* texture,
                               Uint8        r,
                               Uint8        g,
                               Uint8        b)

    # https://wiki.libsdl.org/SDL_UpdateTexture
    int SDL_UpdateTexture(SDL_Texture*    texture,
                          const SDL_Rect* rect,
                          const void*     pixels,
                          int             pitch)
    # https://wiki.libsdl.org/SDL_RenderReadPixels
    int SDL_RenderReadPixels(SDL_Renderer*   renderer,
                             const SDL_Rect* rect,
                             Uint32          format,
                             void*           pixels,
                             int             pitch)
    # https://wiki.libsdl.org/SDL_QueryTexture
    int SDL_QueryTexture(SDL_Texture* texture,
                         Uint32*      format,
                         int*         access,
                         int*         w,
                         int*         h)
    # https://wiki.libsdl.org/SDL_GetRenderTarget
    SDL_Texture* SDL_GetRenderTarget(SDL_Renderer* renderer)
    # https://wiki.libsdl.org/SDL_CreateRGBSurfaceWithFormat
    SDL_Surface* SDL_CreateRGBSurfaceWithFormat(Uint32 flags,
                                            int    width,
                                            int    height,
                                            int    depth,
                                            Uint32 format)
    # https://wiki.libsdl.org/SDL_RenderDrawLine
    # https://wiki.libsdl.org/SDL_RenderDrawLineF
    # https://wiki.libsdl.org/SDL_RenderDrawLines
    # https://wiki.libsdl.org/SDL_RenderDrawLinesF
    # https://wiki.libsdl.org/SDL_RenderDrawPoint
    # https://wiki.libsdl.org/SDL_RenderDrawPointF
    # https://wiki.libsdl.org/SDL_RenderDrawRect
    # https://wiki.libsdl.org/SDL_RenderDrawRectF
    # https://wiki.libsdl.org/SDL_RenderFillRect
    # https://wiki.libsdl.org/SDL_RenderFillRectF
    # https://wiki.libsdl.org/SDL_RenderGeometry
    int SDL_RenderDrawLineF(SDL_Renderer* renderer,
                           float x1,
                           float y1,
                           float x2,
                           float y2)
    int SDL_RenderDrawLinesF(SDL_Renderer* renderer,
                            const SDL_FPoint* points,
                            int count)
    int SDL_RenderDrawPointF(SDL_Renderer* renderer,
                           float x,
                           float y)
    int SDL_RenderDrawRectF(SDL_Renderer* renderer,
                           const SDL_FRect* rect)
    int SDL_RenderFillRectF(SDL_Renderer*   renderer,
                           const SDL_FRect* rect)
    int SDL_RenderGeometry(SDL_Renderer* renderer,
                           SDL_Texture* texture,
                           const SDL_Vertex* vertices,
                           int num_vertices,
                           const int* indices,
                           int num_indices)

    # https://wiki.libsdl.org/SDL_RenderSetScale
    # https://wiki.libsdl.org/SDL_RenderGetScale
    # https://wiki.libsdl.org/SDL_RenderSetLogicalSize
    # https://wiki.libsdl.org/SDL_RenderGetLogicalSize
    # https://wiki.libsdl.org/SDL_RenderGetIntegerScale
    int SDL_RenderSetScale(SDL_Renderer* renderer,
                           float         scaleX,
                           float         scaleY)
    void SDL_RenderGetScale(SDL_Renderer* renderer,
                            float*        scaleX,
                            float*        scaleY)
    int SDL_RenderSetLogicalSize(SDL_Renderer* renderer,
                                 int           w,
                                 int           h)
    void SDL_RenderGetLogicalSize(SDL_Renderer* renderer,
                                  int*          w,
                                  int*          h)
    int SDL_RenderGetIntegerScale(SDL_Renderer* renderer)

    int SDL_VERSION_ATLEAST(int major, int minor, int patch)

    # https://wiki.libsdl.org/SDL_GetWindowPixelFormat
    # https://wiki.libsdl.org/SDL_IntersectRect
    # https://wiki.libsdl.org/SDL_SetRelativeMouseMode
    # https://wiki.libsdl.org/SDL_GetRelativeMouseMode
    # https://wiki.libsdl.org/SDL_GetRenderer
    # https://wiki.libsdl.org/SDL_ConvertSurface
    # https://wiki.libsdl.org/SDL_FreeSurface
    # https://wiki.libsdl.org/SDL_AllocFormat
    # https://wiki.libsdl.org/SDL_FreeFormat
    # https://wiki.libsdl.org/SDL_SetSurfaceBlendMode
    # https://wiki.libsdl.org/SDL_GetSurfaceBlendMode
    Uint32 SDL_GetWindowPixelFormat(SDL_Window* window)
    SDL_bool SDL_IntersectRect(const SDL_Rect* A,
                               const SDL_Rect* B,
                               SDL_Rect*       result)
    int SDL_SetRelativeMouseMode(SDL_bool enabled)
    SDL_bool SDL_GetRelativeMouseMode()
    SDL_Renderer* SDL_GetRenderer(SDL_Window* window)
    SDL_Surface* SDL_ConvertSurface(SDL_Surface * src, const SDL_PixelFormat * fmt, Uint32 flags)
    void SDL_FreeSurface(SDL_Surface * surface)
    SDL_PixelFormat * SDL_AllocFormat(Uint32 pixel_format)
    void SDL_FreeFormat(SDL_PixelFormat *format)
    int SDL_SetSurfaceBlendMode(SDL_Surface * surface, SDL_BlendMode blendMode)
    int SDL_GetSurfaceBlendMode(SDL_Surface * surface, SDL_BlendMode *blendMode)


cdef extern from "pygame.h" nogil:
    ctypedef class pygame.color.Color [object pgColorObject]:
        cdef Uint8 data[4]
        cdef Uint8 len

    ctypedef enum pgColorHandleFlags:
        PG_COLOR_HANDLE_SIMPLE
        PG_COLOR_HANDLE_STR
        PG_COLOR_HANDLE_INT
        PG_COLOR_HANDLE_RESTRICT_SEQ
        PG_COLOR_HANDLE_ALL

    ctypedef class pygame.rect.Rect [object pgRectObject]:
        cdef SDL_Rect r
        cdef object weakreflist

    ctypedef class pygame.window.Window [object pgWindowObject]:
        cdef SDL_Window *_win
        cdef SDL_bool _is_borrowed

    ctypedef struct pgSurfaceObject

    int pgSurface_Check(object surf)
    SDL_Surface* pgSurface_AsSurface(object surf)
    void import_pygame_surface()

    SDL_Window* pg_GetDefaultWindow()
    void import_pygame_base()

    int pgRect_Check(object rect)
    SDL_Rect *pgRect_FromObject(object obj, SDL_Rect *temp)
    object pgRect_New(SDL_Rect *r)
    object pgRect_New4(int x, int y, int w, int h)
    int pgFRect_Check(object rect)
    SDL_FRect *pgFRect_FromObject(object obj, SDL_FRect *temp)
    object pgFRect_New(SDL_FRect *r)
    object pgFRect_New4(float x, float y, float w, float h)
    void import_pygame_rect()

    int pg_RGBAFromObjEx(object color, Uint8 rgba[], pgColorHandleFlags handle_flags) except 0
    object pgColor_NewLength(Uint8 rgba[], Uint8 length)
    void import_pygame_color()
    pgSurfaceObject *pgSurface_New2(SDL_Surface *info, int owner)

    int pgWindow_Check(object win)
    void import_pygame_window()

cdef class Renderer:
    cdef SDL_Renderer* _renderer
    cdef Texture _target
    cdef Window _win
    cdef int _is_borrowed

    cpdef object get_viewport(self)
    cpdef object blit(self, object source, Rect dest=*, Rect area=*, int special_flags=*)

cdef class Texture:
    cdef SDL_Texture* _tex
    cdef readonly Renderer renderer
    cdef readonly int width
    cdef readonly int height

    cpdef void draw(self, srcrect=*, dstrect=*, float angle=*, origin=*,
                    bint flip_x=*, bint flip_y=*)

cdef class Image:
    cdef Color _color
    cdef public float angle
    cdef SDL_Point _origin
    cdef SDL_Point* _originptr
    cdef public bint flip_x
    cdef public bint flip_y
    cdef public float alpha
    cdef public SDL_BlendMode blend_mode

    cdef public Texture texture
    cdef public Rect srcrect

    cpdef void draw(self, srcrect=*, dstrect=*)



import unittest

# A basic unit test to test that pygame.typing can import in python code, and
# the documented attributes are accessible.
# More rigorous testing needs mypy and is therefore implemented in the stubs
# directory.
TYPING_PUBLIC_ATTRS = [
    "RectLike",
    "SequenceLike",
    "FileLike",
    "ColorLike",
    "Point",
    "IntPoint",
]


class TypingTest(unittest.TestCase):
    def test_typing_has_attrs(self):
        import pygame.typing

        for i in TYPING_PUBLIC_ATTRS:
            self.assertTrue(hasattr(pygame.typing, i))


if __name__ == "__main__":
    unittest.main()



from typing import Optional, Union
from typing_extensions import deprecated # added in 3.13

from pygame.typing import Point, RectLike
from pygame.locals import WINDOWPOS_UNDEFINED
from pygame.rect import Rect
from pygame.surface import Surface

def get_grabbed_window() -> Optional[Window]: ...

class Window:
    def __init__(
        self,
        title: str = "pygame window",
        size: Point = (640, 480),
        position: Union[int, Point] = WINDOWPOS_UNDEFINED,
        *,
        fullscreen: bool = ...,
        fullscreen_desktop: bool = ...,
        opengl: bool = ...,
        vulkan: bool = ...,
        hidden: bool = ...,
        borderless: bool = ...,
        resizable: bool = ...,
        minimized: bool = ...,
        maximized: bool = ...,
        mouse_grabbed: bool = ...,
        keyboard_grabbed: bool = ...,
        input_focus: bool = ...,
        mouse_focus: bool = ...,
        allow_high_dpi: bool = ...,
        mouse_capture: bool = ...,
        always_on_top: bool = ...,
        utility: bool = ...,
    ) -> None: ...
    def destroy(self) -> None: ...
    def set_windowed(self) -> None: ...
    def set_fullscreen(self, desktop: bool = False) -> None: ...
    def focus(self, input_only: bool = False) -> None: ...
    def hide(self) -> None: ...
    def show(self) -> None: ...
    def restore(self) -> None: ...
    def maximize(self) -> None: ...
    def minimize(self) -> None: ...
    def set_modal_for(self, parent: Window, /) -> None: ...
    def set_icon(self, icon: Surface, /) -> None: ...
    def get_surface(self) -> Surface: ...
    def flip(self) -> None: ...
    def flash(self, operation: int, /) -> None: ...

    grab_mouse: bool
    grab_keyboard: bool
    title: str
    resizable: bool
    borderless: bool
    always_on_top: bool
    relative_mouse: bool
    opacity: float

    @property
    def mouse_grabbed(self) -> bool: ...
    @property
    def keyboard_grabbed(self) -> bool: ...
    @property
    def focused(self) -> bool: ...
    @property
    def id(self) -> int: ...
    @property
    def mouse_rect(self) -> Optional[Rect]: ...
    @mouse_rect.setter
    def mouse_rect(self, value: Optional[RectLike]) -> None: ...
    @property
    def size(self) -> tuple[int, int]: ...
    @size.setter
    def size(self, value: Point) -> None: ...
    @property
    def minimum_size(self) -> tuple[int, int]: ...
    @minimum_size.setter
    def minimum_size(self, value: Point) -> None: ...
    @property
    def maximum_size(self) -> tuple[int, int]: ...
    @maximum_size.setter
    def maximum_size(self, value: Point) -> None: ...
    @property
    def position(self) -> tuple[int, int]: ...
    @position.setter
    def position(self, value: Union[int, Point]) -> None: ...
    @property
    def opengl(self) -> bool: ...
    @property
    def utility(self) -> bool: ...
    @classmethod
    @deprecated("since 2.4.0. Use either the display module or the Window class with get_surface and flip. Try not to mix display and Window")
    def from_display_module(cls) -> Window: ...



import os
import unittest

from importlib.metadata import version

import pygame

pg_header = os.path.join("src_c", "include", "_pygame.h")


class VersionTest(unittest.TestCase):
    def test_vernum_obj(self):
        major, minor, patch = map(int, pygame.version.ver.split(".")[:3])
        self.assertEqual(pygame.version.vernum.major, major)
        self.assertEqual(pygame.version.vernum[0], major)
        self.assertEqual(pygame.version.vernum.minor, minor)
        self.assertEqual(pygame.version.vernum[1], minor)
        self.assertEqual(pygame.version.vernum.patch, patch)
        self.assertEqual(pygame.version.vernum[2], patch)

    def test_sdl_version(self):
        self.assertEqual(len(pygame.version.SDL), 3)
        self.assertEqual(tuple(pygame.version.SDL), pygame.get_sdl_version())

    def test_installed_version_and_dunder(self):
        self.assertEqual(pygame.__version__, pygame.version.ver)
        self.assertEqual(pygame.__version__, version("pygame-ce"))


if __name__ == "__main__":
    unittest.main()



from collections.abc import Generator, Iterable
from typing import Any, Optional, Union

from pygame.color import Color
from pygame.rect import Rect
from pygame.surface import Surface
from pygame.window import Window as Window

from pygame.typing import ColorLike, RectLike, Point

WINDOWPOS_UNDEFINED: int
WINDOWPOS_CENTERED: int

MESSAGEBOX_ERROR: int
MESSAGEBOX_WARNING: int
MESSAGEBOX_INFORMATION: int

SCALEQUALITY_NEAREST: int
SCALEQUALITY_LINEAR: int
SCALEQUALITY_BEST: int

class RendererDriverInfo:
    name: str
    flags: int
    num_texture_formats: int
    max_texture_width: int
    max_texture_height: int

def get_drivers() -> Generator[RendererDriverInfo, None, None]: ...
def get_grabbed_window() -> Optional[Window]: ...
def messagebox(
    title: str,
    message: str,
    window: Optional[Window] = None,
    info: bool = False,
    warn: bool = False,
    error: bool = False,
    buttons: tuple[str, ...] = ("OK",),
    return_button: int = 0,
    escape_button: int = 0,
) -> int: ...

class Texture:
    def __init__(
        self,
        renderer: Renderer,
        size: Iterable[int],
        static: bool = False,
        streaming: bool = False,
        target: bool = False,
        scale_quality: Optional[int] = None,
    ) -> None: ...
    @staticmethod
    def from_surface(renderer: Renderer, surface: Surface) -> Texture: ...
    renderer: Renderer
    width: int
    height: int
    alpha: int
    blend_mode: int

    @property
    def color(self) -> Color: ...
    @color.setter
    def color(self, value: ColorLike) -> None: ...
    def get_rect(self, **kwargs: Any) -> Rect: ...
    def draw(
        self,
        srcrect: Optional[RectLike] = None,
        dstrect: Optional[RectLike] = None,
        angle: float = 0.0,
        origin: Optional[Iterable[int]] = None,
        flip_x: bool = False,
        flip_y: bool = False,
    ) -> None: ...
    def draw_triangle(
        self,
        p1_xy: Point,
        p2_xy: Point,
        p3_xy: Point,
        p1_uv: Point = (0.0, 0.0),
        p2_uv: Point = (1.0, 1.0),
        p3_uv: Point = (0.0, 1.0),
        p1_mod: Iterable[int] = (255, 255, 255, 255),
        p2_mod: Iterable[int] = (255, 255, 255, 255),
        p3_mod: Iterable[int] = (255, 255, 255, 255),
    ) -> None: ...
    def draw_quad(
        self,
        p1_xy: Point,
        p2_xy: Point,
        p3_xy: Point,
        p4_xy: Point,
        p1_uv: Point = (0.0, 0.0),
        p2_uv: Point = (1.0, 0.0),
        p3_uv: Point = (1.0, 1.0),
        p4_uv: Point = (0.0, 1.0),
        p1_mod: Iterable[int] = (255, 255, 255, 255),
        p2_mod: Iterable[int] = (255, 255, 255, 255),
        p3_mod: Iterable[int] = (255, 255, 255, 255),
        p4_mod: Iterable[int] = (255, 255, 255, 255),
    ) -> None: ...
    def update(self, surface: Surface, area: Optional[RectLike] = None) -> None: ...

class Image:
    def __init__(
        self,
        texture_or_image: Union[Texture, Image],
        srcrect: Optional[RectLike] = None,
    ) -> None: ...
    def get_rect(self) -> Rect: ...
    def draw(
        self, srcrect: Optional[RectLike] = None, dstrect: Optional[RectLike] = None
    ) -> None: ...
    angle: float
    origin: Optional[Iterable[float]]
    flip_x: bool
    flip_y: bool
    alpha: float
    blend_mode: int
    texture: Texture
    srcrect: Rect

    @property
    def color(self) -> Color: ...
    @color.setter
    def color(self, value: ColorLike) -> None: ...

class Renderer:
    def __init__(
        self,
        window: Window,
        index: int = -1,
        accelerated: int = -1,
        vsync: bool = False,
        target_texture: bool = False,
    ) -> None: ...
    @classmethod
    def from_window(cls, window: Window) -> Renderer: ...
    draw_blend_mode: int
    @property
    def draw_color(self) -> Color: ...
    @draw_color.setter
    def draw_color(self, value: ColorLike) -> None: ...
    def clear(self) -> None: ...
    def present(self) -> None: ...
    def get_viewport(self) -> Rect: ...
    def set_viewport(self, area: Optional[RectLike]) -> None: ...
    logical_size: Iterable[int]
    scale: Iterable[float]
    target: Optional[Texture]
    def blit(
        self,
        source: Union[Texture, Image],
        dest: Optional[RectLike] = None,
        area: Optional[RectLike] = None,
        special_flags: int = 0,
    ) -> Rect: ...
    def draw_line(self, p1: Point, p2: Point) -> None: ...
    def draw_point(self, point: Point) -> None: ...
    def draw_rect(self, rect: RectLike) -> None: ...
    def fill_rect(self, rect: RectLike) -> None: ...
    def draw_triangle(
        self, p1: Point, p2: Point, p3: Point
    ) -> None: ...
    def fill_triangle(
        self, p1: Point, p2: Point, p3: Point
    ) -> None: ...
    def draw_quad(
        self, p1: Point, p2: Point, p3: Point, p4: Point
    ) -> None: ...
    def fill_quad(
        self, p1: Point, p2: Point, p3: Point, p4: Point
    ) -> None: ...
    def to_surface(
        self, surface: Optional[Surface] = None, area: Optional[RectLike] = None
    ) -> Surface: ...
    @staticmethod
    def compose_custom_blend_mode(
        color_mode: tuple[int, int, int], alpha_mode: tuple[int, int, int]
    ) -> int: ...
